package net.sf.basedb.clients.web.extensions.list;

import net.sf.basedb.clients.web.extensions.fileviewer.FileViewerUtil;
import net.sf.basedb.clients.web.formatter.AutoItemExportFormatter;
import net.sf.basedb.clients.web.formatter.AutoLinkFormatter;
import net.sf.basedb.clients.web.formatter.FormatterFactory;
import net.sf.basedb.clients.web.formatter.LinkedFileFormatter;
import net.sf.basedb.clients.web.formatter.LinkedItemFormatter;
import net.sf.basedb.core.AnyToAny;
import net.sf.basedb.core.BasicItem;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.File;
import net.sf.basedb.core.Item;
import net.sf.basedb.core.ItemNotFoundException;
import net.sf.basedb.core.Metadata;
import net.sf.basedb.core.Type;
import net.sf.basedb.core.Metadata.PropertyPath;
import net.sf.basedb.core.Nameable;
import net.sf.basedb.util.formatter.FilePathFormatter;
import net.sf.basedb.util.formatter.Formatter;
import net.sf.basedb.util.formatter.NameableFormatter;
import net.sf.basedb.util.formatter.ToStringFormatter;

/**
	A column action implementation that finds an item linked with
	a any-to-any link.
	
	@author nicklas
	@since 3.17
*/
public class AnyLinkColumn
	extends AbstractListColumnBean<BasicItem, Object>
{
	
	/**
		For now, we assume that 'expr' is |linkName|ITEMTYPE|property
		ITEMTYPE and property are optional.
	*/
	@SuppressWarnings({ "unchecked", "rawtypes" })
	public static AnyLinkColumn create(DbControl dc, int index, Item sourceType, String expr, FileViewerUtil fileViewer)
	{
		AnyLinkColumn col = null;
		try
		{
			String[] tmp = expr.split("\\|", 4);
			
			String linkName = tmp[1];
			Item targetType = null;
			if (tmp.length > 2) targetType = Item.valueOf(tmp[2]);
			PropertyPath property = null;
			Item propertyType = targetType;
			if (tmp.length > 3)
			{
				Metadata m = Metadata.getInstance(targetType.getItemClass());
				property = m.getPropertyPath(tmp[3], false);
				if (property.getHibernateType().isEntityType())
				{
					propertyType = Item.fromDataClass(property.getHibernateType().getReturnedClass());
				}
			}
			
			col = new AnyLinkColumn(index, expr, linkName, targetType, property, fileViewer);
			col.setTitle(linkName);
			col.setTooltip((propertyType == null ? "Any" : propertyType.toString()) + " › " + linkName);
			if (propertyType == null || !Nameable.class.isAssignableFrom(propertyType.getItemClass()))
			{
				// If only a link name is provided or the target item is not a Nameable item
				// we can only check if link exists or not
				col.setValueType(Type.BOOLEAN);
				col.setExportFormatter(new AutoItemExportFormatter());
				col.setFormatter((Formatter)new AutoLinkFormatter(fileViewer));
			}
			else if (property == null || property.getHibernateType().isEntityType())
			{
				// The link leads to an item of a specified nameable type -- we filter on the name
				col.setValueType(Type.STRING);
				col.setFilterProperty(expr+(property == null ? "|" : ".") + "name");
				if (propertyType == Item.FILE)
				{
					// Files should export the path and have extra links
					col.setExportFormatter(new FilePathFormatter());
					col.setFormatter((Formatter)new LinkedFileFormatter(fileViewer));
				}
				else
				{
					col.setExportFormatter(new NameableFormatter());
					col.setFormatter((Formatter)new LinkedItemFormatter());
				}
			}
			else
			{
				Type colType = Type.fromHibernateType(property.getHibernateType());
				if (colType != null)
				{
					col.setValueType(colType);
					col.setExportFormatter(FormatterFactory.getTypeFormatter(dc.getSessionControl(), colType));
				}
				else
				{
					// Last fallback -- if we get here the Type.fromHibernateType() should be update with special cases
					col.setValueType(Type.STRING);
					col.setExportFormatter(new ToStringFormatter<>());
				}
				col.setFormatter((Formatter)col.getExportFormatter());
			}
			
		}
		catch (RuntimeException ex)
		{
			ex.printStackTrace(System.out);
		}
		
		return col;
	}

	private final int index;
	private final String linkName;
	private final Item targetType;
	private final PropertyPath<BasicItem, ?> property;
	private final FileViewerUtil fileViewer;
	
	AnyLinkColumn(int index, String id, String linkName, Item targetType, PropertyPath<BasicItem, ?> property, FileViewerUtil fileViewer)
	{
		this.index = index;
		this.linkName = linkName;
		this.targetType = targetType;
		this.property = property;
		this.fileViewer = fileViewer;
		setId(id);
		setProperty(id);
		setExportable(true);
		setFilterable(true);
		setClazz("linkeditemcol");
	}

	@Override
	public final Object getValue(DbControl dc, BasicItem item) 
	{
		Object value = null;
		try
		{
			AnyToAny link = AnyToAny.getByName(dc, item, linkName);
			if (targetType == null || targetType == link.getToType())
			{
				BasicItem target = link.getTo();
				if (fileViewer != null && target instanceof File) 
				{
					fileViewer.getFileContext().setCurrentAnyToAny(item, (File)target, link);
				}
				value = property == null ? target : property.getValue(dc, target);
			}
		}
		catch (ItemNotFoundException ex)
		{}
		
		return value;
	}
	
}