/**
	$Id$

	Copyright (C) 2012 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.clients.web.extensions.list;

import net.sf.basedb.clients.web.extensions.AbstractJspActionFactory;
import net.sf.basedb.clients.web.extensions.JspContext;
import net.sf.basedb.clients.web.formatter.FormatterFactory;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.Item;
import net.sf.basedb.core.Metadata;
import net.sf.basedb.core.RawDataTypes;
import net.sf.basedb.core.ReporterScore;
import net.sf.basedb.core.SessionControl;
import net.sf.basedb.core.Type;
import net.sf.basedb.core.Metadata.PropertyPath;
import net.sf.basedb.core.data.ReporterListScoreData;
import net.sf.basedb.core.plugin.GuiContext;
import net.sf.basedb.util.ClassUtil;
import net.sf.basedb.util.Values;
import net.sf.basedb.util.extensions.InvokationContext;
import net.sf.basedb.util.formatter.Formatter;

/**
	A factory implementation that generates list columns based on a property path
	as used by the {@link Metadata} class. Most properties of the
	{@link ListColumnAction} can be configured by calling setter method for this
	factory. The {@link ListColumnAction#getProperty()} is used as the property
	path in {@link Metadata#getPropertyPath(String, boolean)}. By default, the
	sample property is used for sorting, filtering and exporting, which are 
	enabled by default.
	<p>
	
	A formatter class can be specified by {@link #setFormatterClass(String)} if the given
	class has a public, no-argument constructor. Otherwise a suitable formatter is 
	selected by calling {@link FormatterFactory#getTypeFormatter(SessionControl, Type)}
	with the type taken from {@link ListColumnAction#getValueType()}. 
	
	<p>
	More advanced usage or formatting requirements need a custom factory implementation.
	
	@author Nicklas
	@since 3.2
*/
public class PropertyPathActionFactory<I, V> 
	extends AbstractJspActionFactory<ListColumnAction<I, V>>
{

	private String id;
	private String clazz;
	private String style;
	private String title;
	private String subtitle;
	private String cellClass;
	private String cellStyle;
	private boolean disableOverflowCheck;
	private String tooltip;
	private String property;
	private Type valueType;
	private boolean sortable = true;
	private String sortProperty;
	private boolean filterable = true;
	private String filterProperty;
	private boolean exportable = true;
	private String exportProperty;
	private String formatterClass;
	private String exportFormatterClass;
	
	public PropertyPathActionFactory()
	{}

	/*
		From the ActionFactory interface
		--------------------------------
	*/
	@SuppressWarnings({"unchecked", "rawtypes"})
	@Override
	public ListColumnAction<I, V>[] getActions(InvokationContext<? super ListColumnAction<I, V>> context) 
	{
		JspContext jspContext = (JspContext)context.getClientContext();
		// Get information about what is in the current list
		GuiContext guiContext = jspContext.getGuiContext();
		// The type of item
		Item itemContext = guiContext.getItem();
		// Get the item or data class
		
		Class<I> metaClass = null;
		String entityName = null;
		if (itemContext == Item.RAWDATA)
		{
			// RAWDATA need special handling, since the same class is registered with
			// different names
			metaClass = (Class<I>)itemContext.getDataClass();
			entityName = RawDataTypes.getSafeRawDataType(guiContext.getSubContext()).getEntityName();
		}
		else if (itemContext == Item.REPORTERSCORE)
		{
			// REPORTERSCORE need special handling, since the
			// item and data classer are not BasicItem or BasicData
			metaClass = (Class<I>)ReporterScore.class;
			entityName = ReporterListScoreData.class.getName();
		}
		else
		{
			metaClass = (Class<I>)(itemContext.isBasicItem() ? 
					itemContext.getItemClass() : itemContext.getDataClass());
			// The default entity name (in Hibernate) is the name of the data-layer class
			entityName = itemContext.getDataClass().getName();
		}
			
		Metadata<I> m = Metadata.getInstance(metaClass, entityName);
		PropertyPath<I, V> propertyPath = m.getPropertyPath(property, false);
		PropertyPath<I, ?> exportPath = exportProperty == null ? propertyPath : m.getPropertyPath(exportProperty, false);
		PropertyPathAction<I, V> action = new PropertyPathAction<I, V>(propertyPath, exportPath);
		action.setId(id);
		action.setClazz(clazz);
		action.setStyle(style);
		action.setTitle(title);
		action.setSubtitle(subtitle);
		action.setCellClass(cellClass);
		action.setCellStyle(cellStyle);
		action.setDisableOverflowCheck(disableOverflowCheck);
		action.setTooltip(tooltip);
		action.setProperty(property);
		action.setValueType(valueType);
		action.setSortable(sortable);
		action.setSortProperty(sortProperty);
		action.setFilterable(filterable);
		action.setFilterProperty(filterProperty);
		action.setExportable(exportable);
		action.setExportProperty(exportProperty);
		action.setFormatter(getFormatter(context, formatterClass));
		if (exportable) action.setExportFormatter(getFormatter(context, exportFormatterClass));
		return new ListColumnAction[] { action };
	}
	// ----------------------------------
	

	/*
		Factory initializtion methods
		see ListColumnAction for more information
		-----------------------------
	*/
	public void setId(String id)
	{
		this.id = id;
	}
	
	public void setClazz(String clazz)
	{
		this.clazz = clazz;
	}
	
	public void setStyle(String style)
	{
		this.style = style;
	}
	
	public void setTitle(String title)
	{
		this.title = title;
	}

	/**
		@since 3.18
	*/
	public void setSubtitle(String subtitle)
	{
		this.subtitle = subtitle;
	}

	/**
		@since 3.18
	*/
	public void setCellClass(String cellClass)
	{
		this.cellClass = cellClass;
	}

	/**
		@since 3.18
	*/
	public void setCellStyle(String cellStyle)
	{
		this.cellStyle = cellStyle;
	}
	
	/**
		@since 3.18
	*/
	public void setDisableOverflowCheck(boolean disableOverflowCheck)
	{
		this.disableOverflowCheck = disableOverflowCheck;
	}
	
	public void setTooltip(String tooltip)
	{
		this.tooltip = tooltip;
	}

	public void setProperty(String property)
	{
		this.property = property;
	}

	public void setValueType(String valueType)
	{
		this.valueType = Type.valueOf(valueType);
	}

	public void setSortable(String sortable)
	{
		this.sortable = Values.getBoolean(sortable);
	}

	public void setSortProperty(String sortProperty)
	{
		this.sortProperty = sortProperty;
	}

	public void setFilterable(String filterable)
	{
		this.filterable = Values.getBoolean(filterable);
	}

	public void setFilterProperty(String filterProperty)
	{
		this.filterProperty = filterProperty;
	}
	
	public void setExportable(String exportable)
	{
		this.exportable = Values.getBoolean(exportable);
	}

	public void setExportProperty(String exportProperty)
	{
		this.exportProperty = exportProperty;
	}
	
	public void setFormatterClass(String formatterClass)
	{
		this.formatterClass = formatterClass;
	}
	
	public void setExportFormatterClass(String formatterClass)
	{
		this.exportFormatterClass = formatterClass;
	}
	// ------------------------------------

	/**
		Create a formatter instance that should be used for the online table. If
		a className is specified try to load an instance
		of that class, otherwise use {@link FormatterFactory#getTypeFormatter(SessionControl, Type)}
		with the value type from {@link #setValueType(String)}.
	*/
	@SuppressWarnings({ "unchecked", "rawtypes" })
	protected Formatter<V> getFormatter(InvokationContext context, String className)
	{
		Formatter<V> formatter = null;
		if (className != null)
		{
			try
			{
				formatter = (Formatter<V>)ClassUtil.checkAndLoadClass(context.getClassLoader(), className, true, Formatter.class).getDeclaredConstructor().newInstance();
			}
			catch (Exception ex)
			{
				throw new RuntimeException(ex);
			}
		}
		if (formatter == null)
		{
			formatter = (Formatter<V>)FormatterFactory.getTypeFormatter(context.getClientContext().getSessionControl(), valueType);
		}
		return formatter;
	}
	
	
	/**
		A concrete implementation of the ListColumnAction
		interface. We subclass the {@link AbstractListColumnBean}
		and then use the {@link PropertyPath} in the {@link #getValue(DbControl, Object)}
		and {@link #getExportValue(DbControl, Object)} methods.
	*/
	static class PropertyPathAction<I, V>
		extends AbstractListColumnBean<I, V>
	{
		private final PropertyPath<I, V> path;
		private final PropertyPath<I, ?> exportPath;
		
		PropertyPathAction(PropertyPath<I, V> path, PropertyPath<I, ?> exportPath)
		{
			this.path = path;
			this.exportPath = exportPath;
		}
		
		/*
			From the ListColumnAction interface
			-----------------------------------
		*/
		@Override
		public V getValue(DbControl dc, I item) 
		{
			return path.getValue(dc, item);
		}
		@Override
		public Object getExportValue(DbControl dc, I item) 
		{
			return exportPath.getValue(dc, item);
		}
		// ------------------------------------

		
	}
	
}
