/*
	$Id: Data.java 7703 2019-04-11 07:31:26Z nicklas $

	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Häkkinen, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.clients.web.taglib.table;

import javax.servlet.jsp.JspException;
import javax.servlet.jsp.JspTagException;

import net.sf.basedb.clients.web.taglib.StylableTag;

/**
	This tag begins the data section of the table.

	<p>
	<b>Syntax:</b>
	<pre>
   &lt;tbl:data
      id=...
      clazz=...
      style=...
      subclass=...
   &gt;
</pre>


	<table class="as-grid">
	<caption>Tag attributes</caption>
	<tr>
		<td><b>Attribute</b></td>
		<td><b>Default value</b></td>
		<td><b>Required</b></td>
		<td><b>Description</b></td>
	</tr>
	<tr>
		<td>id</td>
		<td>-</td>
		<td>yes</td>
		<td>
			The ID of the data section. The value of this attribute
			goes directly into the standard HTML <code>id</code> attribute.
			This means that if you need a reference to the table object in your own
			JavaScript code, you may for example use the following code:
			<pre class="code">
data = document.getElementById('&lt;id&gt;');
</pre>
		</td>
	</tr>
	<tr>
		<td>clazz</td>
		<td>data</td>
		<td>no</td>
		<td>
			The value if this attribute goes directly into the standard HTML 
			<code>class</code> attribute. This allows you to apply different styles
			to different tables.
		</td>
	</tr>
	<tr>
		<td>style</td>
		<td>-</td>
		<td>no</td>
		<td>
			The value if this attribute goes directly into the standard HTML 
			<code>style</code> attribute. This allows you to apply different styles
			to different tables.
		</td>
	</tr>
	</table>
	<p>
	The <code>&lt;tbl:data&gt;</code> may contain the following subtags:
	<ul>
	<li>{@link Headers &lt;tbl:headers&gt;}: Placeholder for column headers
	<li>{@link Rows &lt;tbl:rows&gt;}: Placeholder for rows
	</ul>

	@author Nicklas
	@version 2.0
*/
public class Data
	extends StylableTag
{

	/**
		
	*/
	private static final long serialVersionUID = 6633768910320305875L;

	public Data()
	{
		super("data");
	}
	
	/**
		The parent &lt;tbl:table&gt; tag.
	*/
	private Table table;

	/*
		Taglib initialization methods
	*/

	/*
		Package methods
	*/
	Table getTable()
	{
		return table;
	}

	/*
		Display methods
	*/
	@Override
	public int doStartTag()
		throws JspException
	{
		if (!(getParent() instanceof Table)) throw new JspTagException("Tag <tbl:data> must be inside a <tbl:table> tag");
		table = (Table)getParent();
		table.endColumnDefinitions();

		StringBuilder sb = new StringBuilder();
		sb.append("<div ");
		addIdAndStyles(sb);
		addDynamicAttributes(sb);
		sb.append("><table>\n");
		try
		{
			pageContext.getOut().print(sb.toString());
		}
		catch (Exception ex)
		{
			throw new JspTagException(ex.getMessage());
		}
		return EVAL_BODY_INCLUDE;
	}

	@Override
	public int doEndTag()
		throws JspException
	{
		StringBuilder sb = new StringBuilder();
		sb.append("</table></div>\n");
		try
		{
			pageContext.getOut().print(sb.toString());
		}
		catch (Exception ex)
		{
			throw new JspTagException(ex.getMessage());
		}
		return EVAL_PAGE;
	}


}

