/**
	$Id: DerivedBioAssay.java 7927 2021-03-26 07:26:24Z nicklas $

	Copyright (C) 2011 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Date;
import java.util.HashSet;
import java.util.IdentityHashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;

import net.sf.basedb.core.Transactional.Action;
import net.sf.basedb.core.data.ArrayBatchData;
import net.sf.basedb.core.data.ArrayDesignData;
import net.sf.basedb.core.data.ArraySlideData;
import net.sf.basedb.core.data.DerivedBioAssayData;
import net.sf.basedb.core.data.ExtractData;
import net.sf.basedb.core.data.HardwareData;
import net.sf.basedb.core.data.PhysicalBioAssayData;
import net.sf.basedb.core.data.ProtocolData;
import net.sf.basedb.core.data.SoftwareData;
import net.sf.basedb.core.hibernate.TypeWrapper;
import net.sf.basedb.core.query.Hql;
import net.sf.basedb.core.query.Restrictions;

/**

	@author Nicklas
	@since 3.0
	@base.modified $Date: 2021-03-26 08:26:24 +0100 (Fri, 26 Mar 2021) $
*/
public class DerivedBioAssay
	extends AnnotatedItem
	implements Registered, FileStoreEnabled, Subtypable, Listable
{

	/**
		The type of item represented by this class.
		@see Item#DERIVEDBIOASSAY
		@see #getType()
	*/
	public static final Item TYPE = Item.DERIVEDBIOASSAY;
	
	/**
		The id for the <code>DerivedBioAssayType</code> item representing a
		scanned hybridization.
	*/
	public static final String SCAN = "net.sf.basedb.core.DerivedBioAssayType.SCAN";

	
	/**
		The id for the <code>DerivedBioAssayType</code> item representing
		original sequenced data.
	*/
	public static final String SEQUENCES = "net.sf.basedb.core.DerivedBioAssayType.SEQUENCES";
	
	/**
		The id for the <code>DerivedBioAssayType</code> item representing
		sequenced data that has been aligned to a reference database.
	*/
	public static final String ARRANGEMENT = "net.sf.basedb.core.DerivedBioAssayType.ARRANGEMENT";

	
	/**
		Get a <code>DerivedBioAssay</code> item when you know the id.
	
		@param dc The <code>DbControl</code> which will be used for
			permission checking and database access.
		@param id The id of the item to load
		@return The <code>DerivedBioAssay</code> item
		@throws ItemNotFoundException If an item with the specified 
			id is not found
		@throws PermissionDeniedException If the logged in user doesn't 
			have read permission to the item
		@throws BaseException If there is another error
	*/
	public static DerivedBioAssay getById(DbControl dc, int id)
		throws ItemNotFoundException, PermissionDeniedException, BaseException
	{
		DerivedBioAssay dbas = dc.loadItem(DerivedBioAssay.class, id);
		if (dbas == null) throw new ItemNotFoundException("DerivedBioAssay[id="+id+"]");
		return dbas;
	}
	
	/**
		Create a new parent-less derived bioassay. Parent items can be 
		added by calling {@link #addPhysicalBioAssay(PhysicalBioAssay)} or
		{@link #addParent(DerivedBioAssay)}.
		@param dc The <code>DbControl</code> which will be used for
			permission checking and database access
		@param isRoot TRUE if the new item is a root derived bioassay
			(eg. it have physical bioassays as parent items)
		@param job The job that created the new bioassay
		@return A new DerivedBioAssay item
		@since 3.2
	*/
	public static DerivedBioAssay getNew(DbControl dc, boolean isRoot, Job job)
	{
		DerivedBioAssay dbas = dc.newItem(DerivedBioAssay.class);
		dbas.setName("New derived bioassay");
		dbas.getData().setRoot(isRoot);
		dbas.getData().setEntryDate(new Date());
		dbas.getData().setJob(job == null ? null : job.getData());
		return dbas;
		
	}
	
	/**
		Create a new (root) derived bioassay from a physical bioassay.
		@param dc The <code>DbControl</code> which will be used for
			permission checking and database access
		@param bioAssay The parent physical bioassay
		@param job The job that created the new bioassay
		@return A new DerivedBioAssay item
	*/
	public static DerivedBioAssay getNew(DbControl dc, PhysicalBioAssay bioAssay, Job job)
	{
		DerivedBioAssay dbas = getNew(dc, true, job);
		dbas.addPhysicalBioAssay(bioAssay);
		return dbas;
	}
	
	/**
		Create a new child derived bioassay from a parent derived bioassay.
		@param dc The <code>DbControl</code> which will be used for
			permission checking and database access
		@param parent The parent bioassay 
		@param job The job that created the new bioassay
		@return A new DerivedBioAssay item
	*/
	public static DerivedBioAssay getNew(DbControl dc, DerivedBioAssay parent, Job job)
	{
		DerivedBioAssay dbas = getNew(dc, false, job);
		dbas.addParent(parent);
		return dbas;
	}
	
	/**
		Get a query that returns derived bioassay.
		@return An {@link ItemQuery} object
	*/
	public static ItemQuery<DerivedBioAssay> getQuery()
	{
		return new ItemQuery<DerivedBioAssay>(DerivedBioAssay.class);
	}
	
	DerivedBioAssay(DerivedBioAssayData data)
	{
		super(data);
	}
	
	@Override
	DerivedBioAssayData getData()
	{
		return (DerivedBioAssayData)getBasicData();
	}
	
	/*
		From the Identifiable interface
		-------------------------------------------
	*/
	@Override
	public Item getType()
	{
		return TYPE;
	}
	// -------------------------------------------

	/*
		From the BasicItem class
		-------------------------------------------
	*/
	/**
	 	Get all:
	 	<ul>
	 	<li>Child derived bioassays
	 	<li>Child raw bioassays
	 	</ul>
	*/
	@Override
	public Set<ItemProxy> getUsingItems()
	{
		Set<ItemProxy> using = super.getUsingItems();
		org.hibernate.Session session = getDbControl().getHibernateSession();
		
		// Child derived bioassays
		org.hibernate.query.Query<Integer> query = HibernateUtil.getPredefinedQuery(session, 
			"GET_CHILD_DERIVEDBIOASSAYS_FOR_DERIVEDBIOASSAY", Integer.class, "dba.id");
			/*
				SELECT {1}
				FROM DerivedBioAssayData dba
				WHERE dba.parent = :bioAssay
			*/
		query.setParameter("bioAssay", getId(), TypeWrapper.H_INTEGER);
		addUsingItems(using, Item.DERIVEDBIOASSAY, query);

		// Child raw bioassays
		query = HibernateUtil.getPredefinedQuery(session, 
			"GET_RAWBIOASSAYS_FOR_DERIVEDBIOASSAY", Integer.class, "rba.id");
			/*
				SELECT {1}
				FROM RawBioAssayData rba
				WHERE rba.parentBioAssay = :bioAssay
			*/
		query.setParameter("bioAssay", getId(), TypeWrapper.H_INTEGER);
		addUsingItems(using, Item.RAWBIOASSAY, query);
		
		return using;
	}
	
	/**
		
	*/
	@Override
	public boolean isUsed()
		throws BaseException
	{
		org.hibernate.Session session = getDbControl().getHibernateSession();
		boolean used = false;
		
		if (!used)
		{
			org.hibernate.query.Query<Long> query = HibernateUtil.getPredefinedQuery(session, 
				"GET_RAWBIOASSAYS_FOR_DERIVEDBIOASSAY", Long.class, "count(*)");
			/*
				SELECT {1}
				FROM RawBioAssayData rba
				WHERE rba.parentBioAssay = :bioAssay
			*/
			query.setParameter("bioAssay", getId(), TypeWrapper.H_INTEGER);
			used = HibernateUtil.loadData(query) > 0;
		}

		if (!used)
		{
			org.hibernate.query.Query<Long> query = HibernateUtil.getPredefinedQuery(session, 
				"GET_CHILD_DERIVEDBIOASSAYS_FOR_DERIVEDBIOASSAY", Long.class, "count(*)");
			/*
				SELECT {1}
				FROM DerivedBioAssayData dba
				INNER JOIN dba.parents p
				WHERE p = :bioAssay
			*/
			query.setParameter("bioAssay", getId(), TypeWrapper.H_INTEGER);
			used = HibernateUtil.loadData(query) > 0;
		}
		
		return used || super.isUsed();
	}
	
	/**
		Any parent bioassays found among the given items are removed from
		this item. NOTE! Due to {@link FixDerivedBioAssayParentsRecursivelyAction}
		it should not be possible to create circular references between derived
		bioassays, but in case something changes in the future we have implemented
		this method anyway.
		@since 3.8
	*/
	@Override
	int breakCircularReferences(Set<ItemProxy> items)
	{
		int removed = super.breakCircularReferences(items);
		DbControl dc = getDbControl();
		Set<DerivedBioAssayData> parents = getData().getParents();
		for (ItemProxy p : items)
		{
			if (DerivedBioAssay.class.isAssignableFrom(p.getType().getItemClass()))
			{
				DerivedBioAssay dba = (DerivedBioAssay)p.getItem(dc);
				if (parents.remove(dba.getData()))
				{
					removed++;
				}
			}
		}
		return removed;
	}

	/**
		Automatically try to assign an extract from parent items at
		creation time.
	*/
	@Override
	void onBeforeCommit(Action action)
	{
		super.onBeforeCommit(action);
		
		if (action == Action.CREATE && !extractHasBeenSet && !isRoot())
		{
			ExtractData extract = null;
			for (DerivedBioAssayData parent : getData().getParents())
			{
				ExtractData e = parent.getExtract();
				if (extract == null)
				{
					extract = e;
				}
				else if (e != null && !extract.equals(e))
				{
					// Different extracts on different parents --> do not assign
					extract = null;
					break;
				}
			}
			if (extract != null) getData().setExtract(extract);
		}
		
	}

	/**
		Set protocol, hardware and software from project default settings.
	*/
	@Override
	void setProjectDefaults(Project activeProject)
		throws BaseException
	{
		super.setProjectDefaults(activeProject);
		if (!hasPermission(Permission.WRITE)) return;
			
		DbControl dc = getDbControl();
		if (!protocolHasBeenSet)
		{
			ProtocolData protocol = 
				(ProtocolData)activeProject.findDefaultRelatedData(dc, this, Item.PROTOCOL, false);
			if (protocol != null)
			{
				getData().setProtocol(protocol);
				protocolHasBeenSet = true;
			}
		}
		if (!hardwareHasBeenSet)
		{
			HardwareData hardware = 
				(HardwareData)activeProject.findDefaultRelatedData(dc, this, Item.HARDWARE, false);
			if (hardware != null) 
			{
				getData().setHardware(hardware);
				hardwareHasBeenSet = true;
			}
		}
		if (!softwareHasBeenSet)
		{
			SoftwareData software = 
				(SoftwareData)activeProject.findDefaultRelatedData(dc, this, Item.SOFTWARE, false);
			if (software != null) 
			{
				getData().setSoftware(software);
				softwareHasBeenSet = true;
			}
		}
	}
	// -------------------------------------------
	/*
		From the Annotatable interface
		-------------------------------------------
	*/
	/**
		Get the extract and parent bioassays or physical bioassays.
	*/
	@Override
	public Set<Annotatable> getAnnotatableParents()
		throws BaseException
	{
		Set<Annotatable> annotatable = new HashSet<Annotatable>();
		DerivedBioAssayData data = getData();
		DbControl dc = getDbControl();
		
		if (isRoot())
		{
			ItemQuery<PhysicalBioAssay> query = getPhysicalBioAssays();
			query.include(Include.ALL);
			annotatable.addAll(query.list(dc));
		}
		else
		{
			ItemQuery<DerivedBioAssay> query = getParents();
			query.include(Include.ALL);
			annotatable.addAll(query.list(dc));
		}
		
		addAnnotatableParents(dc, annotatable, 
			data.getExtract(),
			data.getHardware(),
			data.getSoftware(),
			data.getProtocol(),
			data.getKit()
		);
		
		return annotatable;
	}
	@Override
	public Protocol getProtocol()
	{
		return getDbControl().getItem(Protocol.class, getData().getProtocol());
	}
	// -------------------------------------------
	/*
		From the Subtypable interface
		-----------------------------
	*/
	@Override
	@SubtypableRelatedItems({Item.PHYSICALBIOASSAY, Item.DERIVEDBIOASSAY, Item.EXTRACT, Item.SOFTWARE, Item.HARDWARE, Item.PROTOCOL, Item.KIT})
	public ItemSubtype getItemSubtype()
	{
		return getDbControl().getItem(ItemSubtype.class, getData().getItemSubtype());
	}
	@Override
	public void setItemSubtype(ItemSubtype subtype)
	{
		checkPermission(Permission.WRITE);
		if (subtype != null)
		{
			subtype.setOnItem(this);
		}
		else
		{
			getData().setItemSubtype(null);
		}
	}
	// -------------------------------------------
	/*
		From the Registered interface
		-------------------------------------------
	*/
	@Override
	public Date getEntryDate()
	{
		return DateUtil.copy(getData().getEntryDate());
	}
	@Override
	public void setEntryDate(Date entryDate)
	{
		if (isInDatabase()) 
		{
			throw new PermissionDeniedException("Not allowed to change entryDate of " + this);
		}
		getData().setEntryDate(DateUtil.setDateOrToday(entryDate));
	}
	// -------------------------------------------

	/*
		From the FileStoreEnabled interface
		-------------------------------------------
	*/
	@Override
	public FileSet getFileSet()
	{
		DbControl dc = getDbControl();
		FileSet fs = dc.getItem(FileSet.class, getData().getFileSet(), this);
		if (fs == null)
		{
			fs = FileSet.getNew(dc, this);
			getData().setFileSet(fs.getData());
			dc.saveItemIf(this, fs, false);
		}
		return fs;
	}
	@Override
	public boolean hasFileSet()
	{
		return getData().getFileSet() != null;
	}
	@Override
	public Platform getPlatform()
	{
		return null;
	}
	@Override
	public PlatformVariant getVariant()
	{
		return null;
	}
	@Override
	public Collection<FileSet> getParentFileSets()
	{
		ItemQuery<DerivedBioAssay> query = getParents();
		query.include(Include.ALL);
		Collection<FileSet> parents = null;
		for (DerivedBioAssay parent : query.list(getDbControl()))
		{
			if (parent.hasFileSet())
			{
				if (parents == null) parents = new ArrayList<FileSet>();
				parents.add(parent.getFileSet());
			}
		}
		return parents;
	}
	// -------------------------------------------

	public void testFixChildren()
	{
		FixDerivedBioAssayParentsRecursivelyAction.get(getDbControl()).add(getData());
	}
	
	/**
		Add a physical bioassay as a parent to this derived bioassay.
		Note! Physical bioassays can only be added if this derived bioassay
		is a root bioassay. The physical bioassay for non-root derived
		bioassays are automatically assigned.
		@param bioAssay The parent bioassay to add 
		@since 3.2
	*/
	public void addPhysicalBioAssay(PhysicalBioAssay bioAssay)
		throws PermissionDeniedException, InvalidDataException
	{
		checkPermission(Permission.WRITE);
		if (!isRoot()) throw new PermissionDeniedException("Can't add physical bioassay to non-root derived bioassay");
		if (bioAssay == null) throw new InvalidUseOfNullException("bioAssay");
		bioAssay.checkPermission(Permission.USE);
		boolean wasAdded = getData().getPhysicalBioAssays().add(bioAssay.getData());
		if (wasAdded)
		{
			FixDerivedBioAssayParentsRecursivelyAction.get(getDbControl()).add(getData());
		}
	}
	
	/**
		Remove a physical bioassay as a parent item for this derived
		bioassay.
		Note! Physical bioassays can only be removed if this derived bioassay
		is a root bioassay. The physical bioassay for non-root derived
		bioassays are automatically assigned.
		@param bioAssay The parent bioassay to remove from this bioassay
		@since 3.2
	*/
	public void removePhysicalBioAssay(PhysicalBioAssay bioAssay)
		throws PermissionDeniedException, InvalidDataException
	{
		checkPermission(Permission.WRITE);
		if (!isRoot()) throw new PermissionDeniedException("Can't remove physical bioassay from non-root derived bioassay");
		if (bioAssay == null) throw new InvalidUseOfNullException("bioAssay");
		boolean wasRemoved = getData().getPhysicalBioAssays().remove(bioAssay.getData());
		if (wasRemoved)
		{
			FixDerivedBioAssayParentsRecursivelyAction.get(getDbControl()).add(getData());
		}
	}
	
	/**
		Get a query that returns the physical bioassays used to create
		this derived bioassay.
		@return An <code>ItemQuery</code> object
		@since 3.2
	*/
	public ItemQuery<PhysicalBioAssay> getPhysicalBioAssays()
	{
		ItemQuery<PhysicalBioAssay> query = PhysicalBioAssay.getQuery();
		query.joinPermanent(Hql.innerJoin("derivedBioAssays", Item.DERIVEDBIOASSAY.getAlias()));
		query.restrictPermanent(
			Restrictions.eq(
				Hql.alias(Item.DERIVEDBIOASSAY.getAlias()), 
				Hql.entity(this)
			)
		);
		return query;
	}

	/**
		Add a derived bioassay as a parent to this derived bioassay.
		Note! Derived bioassays can only be added if this derived bioassay
		is not a root bioassay. The physical bioassays are automatically assigned.
		@param bioAssay The parent bioassay to add 
		@since 3.2
	*/
	public void addParent(DerivedBioAssay bioAssay)
		throws PermissionDeniedException, InvalidDataException
	{
		checkPermission(Permission.WRITE);
		if (isRoot()) throw new PermissionDeniedException("Root derived bioassays can't have parent derived bioassays");
		if (bioAssay == null) throw new InvalidUseOfNullException("bioAssay");
		if (this.equals(bioAssay)) throw new InvalidDataException("Can't add self as parent: " + this);
		bioAssay.checkPermission(Permission.USE);
		boolean wasAdded = getData().getParents().add(bioAssay.getData());
		if (wasAdded)
		{
			FixDerivedBioAssayParentsRecursivelyAction.get(getDbControl()).add(getData());
		}
	}

	/**
		Remove a derived bioassay as a parent item for this derived
		bioassay.
		Note! Derived bioassays can only be used as parents if this derived bioassay
		is not a root bioassay. The physical bioassays are automatically assigned.
		@param bioAssay The parent bioassay to remove from this bioassay
		@since 3.2
	*/
	public void removeParent(DerivedBioAssay bioAssay)
		throws PermissionDeniedException, InvalidDataException
	{
		checkPermission(Permission.WRITE);
		if (isRoot()) throw new PermissionDeniedException("Root derived bioassays can't have parent derived bioassays");
		if (bioAssay == null) throw new InvalidUseOfNullException("bioAssay");
		boolean wasRemoved = getData().getParents().remove(bioAssay.getData());
		if (wasRemoved)
		{
			FixDerivedBioAssayParentsRecursivelyAction.get(getDbControl()).add(getData());
		}
	}

	
	/**
		Get all parent derived bioassays.
		@return A query returning all parent bioassays
		@since 3.2
	*/
	public ItemQuery<DerivedBioAssay> getParents()
	{
		ItemQuery<DerivedBioAssay> query = DerivedBioAssay.getQuery();
		query.joinPermanent(Hql.innerJoin("children", "c"));
		query.restrictPermanent(
			Restrictions.eq(
				Hql.alias("c"),
				Hql.entity(this)
			)
		);
		return query;
	}

	
	/**
		Check if this bioassay set is derived from one or more physical bioassays or 
		from derived bioassays. A root derived bioassay can only have physical bioassays
		as parent items. A non-root derived bioassay can only have other derived
		bioassays as parents, but the core keep automatically keep track of the
		physical bioassays.
		@return TRUE if this is a root bioassay set derived form a physical bioassay
	*/
	public boolean isRoot()
	{
		return getData().isRoot();
	}
	
	/**
		Get the extract that is the parent of the data in this
		derived bioassay. The extract should be an extract that is
		present on the physical bioassay that this bioassay is derived from.
		But this is not enforced by the core. If no extract is specified this
		bioassay applies to all extracts that are part of the physical bioassay.
		@return An extract or null
	*/
	public Extract getExtract()
	{
		return getDbControl().getItem(Extract.class, getData().getExtract());
	}
	
	private boolean extractHasBeenSet;
	
	/**
		Set the extract that is the parent of data in this bioassay. The extract 
		should be an extract that is present on the physical bioassay that this 
		bioassay is derived from. But this is not enforced by the core.
		@param extract The extract or null
	*/
	public void setExtract(Extract extract)
	{
		checkPermission(Permission.WRITE);
		if (extract != null) extract.checkPermission(Permission.USE);
		extractHasBeenSet = true;
		getData().setExtract(extract == null ? null : extract.getData());
	}
	
	/**
		Get all child derived bioassays.
		@return A query returning all child bioassays
	*/
	public ItemQuery<DerivedBioAssay> getChildren()
	{
		ItemQuery<DerivedBioAssay> query = DerivedBioAssay.getQuery();
		query.joinPermanent(Hql.innerJoin("parents", "p"));
		query.restrictPermanent(
			Restrictions.eq(
				Hql.alias("p"),
				Hql.entity(this)
			)
		);
		return query;
	}

	/**
		Get the job that created this bioassay.
		@return A job or null if not created by a job
	*/
	public Job getJob()
	{
		return getDbControl().getItem(Job.class, getData().getJob());
	}
	
	/**
		Set the job on this derived bioassay that was used to create or import
		data for it.
	
		@param job The job or null to clear the existing job
		@throws PermissionDeniedException If the logged in user doesn't have 
			write permission
		@since 3.18.1
	*/
	public void setJob(Job job)
		throws PermissionDeniedException
	{
		checkPermission(Permission.WRITE);
		if (job != null) job.checkPermission(Permission.USE);
		getData().setJob(job == null ? null : job.getData());
	}
	
	private boolean protocolHasBeenSet = false;
	/**
		Set the {@link Protocol} that describes the procedure used when 
		creating this bioassay set.
		@param protocol A <code>Protocol</code> item, or null if not known
		@throws PermissionDeniedException If the logged in user doesn't have
			write permission on this event or use permission for the protocol
	*/
	public void setProtocol(Protocol protocol)
		throws PermissionDeniedException
	{
		checkPermission(Permission.WRITE);
		if (protocol != null) protocol.checkPermission(Permission.USE);
		protocolHasBeenSet = true;
		getData().setProtocol(protocol == null ? null : protocol.getData());
	}
	
	/**
		Get the {@link Kit} that was used when creating this bioassay.
		@return A <code>Kit</code> item, or null if not known
		@throws PermissionDeniedException If the logged in user doesn't have
			read permission to the kit
		@throws BaseException If there is another error
		@since 3.7
	*/
	public Kit getKit()
		throws PermissionDeniedException, BaseException
	{
		return getDbControl().getItem(Kit.class, getData().getKit());
	}
	
	/**
		Set the {@link Kit} that was used when creating this bioassay.
		@param kit A <code>Kit</code> item, or null if not known
		@throws PermissionDeniedException If the logged in user doesn't have
			write permission on this event or use permission for the kit
		@since 3.7
	*/
	public void setKit(Kit kit)
		throws PermissionDeniedException
	{
		checkPermission(Permission.WRITE);
		if (kit != null) kit.checkPermission(Permission.USE);
		getData().setKit(kit == null ? null : kit.getData());
	}
	
	/**
		Get the {@link Hardware} that was used  when 
		creating this bioassay set.
	 	@return A <code>Hardware</code> item, or null if not known
	*/
	public Hardware getHardware()
	{
		return getDbControl().getItem(Hardware.class, getData().getHardware());
	}
	
	private boolean hardwareHasBeenSet = false;
	/**
	 	Set the {@link Hardware} that was used when 
		creating this bioassay set.
	 	@param hardware A <code>Hardware</code> item, or null if not known
	 	@throws PermissionDeniedException If the logged in user doesn't have
	 		write permission on this event or use permission for the hardware
	*/
	public void setHardware(Hardware hardware)
		throws PermissionDeniedException
	{
		checkPermission(Permission.WRITE);
		if (hardware != null) hardware.checkPermission(Permission.USE);
		hardwareHasBeenSet = true;
		getData().setHardware(hardware == null ? null : hardware.getData());
	}
	
	/**
		Get the {@link Software} that was used when 
		creating this bioassay set.
	 	@return A <code>Software</code> item, or null if not known
	*/
	public Software getSoftware()
	{
		return getDbControl().getItem(Software.class, getData().getSoftware());
	}
	
	private boolean softwareHasBeenSet = false;
	/**
	 	Set the {@link Software} that was used when 
		creating this bioassay set.
	 	@param software A <code>Software</code> item, or null if not known
	 	@throws PermissionDeniedException If the logged in user doesn't have
	 		write permission on this event or use permission for the software
	*/
	public void setSoftware(Software software)
		throws PermissionDeniedException
	{
		checkPermission(Permission.WRITE);
		if (software != null) software.checkPermission(Permission.USE);
		softwareHasBeenSet = true;
		getData().setSoftware(software == null ? null : software.getData());
	}

	
	public ItemQuery<RawBioAssay> getRawBioAssays()
	{
		ItemQuery<RawBioAssay> query = RawBioAssay.getQuery();
		query.restrictPermanent(
			Restrictions.eq(
				Hql.property("parentBioAssay"),
				Hql.entity(this)
			)
		);
		return query;
	}
	
	/**
		Get the array design connected to this derived bioassay by 
		following the path from the physical bioassay to the array slide to 
		the array batch  to the array design. If this derived bioasay
		has multiple parents that are linked to different array designs,
		null is returned.
		@return The array design, or null if no array design is connected
		@throws PermissionDeniedException If the logged in user doesn't have 
			read permission for the array design
		@throws BaseException If there is another error.
	*/
	public ArrayDesign getArrayDesign()
		throws PermissionDeniedException, BaseException
	{
		ArrayDesignData uniqueDesign = null;
		for (PhysicalBioAssayData pba : getData().getPhysicalBioAssays())
		{
			ArraySlideData slide = pba.getArraySlide();
			if (slide == null) continue;
			
			ArrayBatchData batch = slide.getArrayBatch();
			if (batch == null) continue;
			
			ArrayDesignData design = batch.getArrayDesign();
			if (design == null) continue;
			
			if (uniqueDesign == null)
			{
				// First array design we see is (temporarily) accepted as the one to return
				uniqueDesign = design;
			}
			else
			{
				// If the design are different, break out and return null
				if (!design.equals(uniqueDesign)) return null;
			}
		}
		
		return getDbControl().getItem(ArrayDesign.class, uniqueDesign);
	}

	/**
		Counts all raw bioassays that have been created from this bioassay. The count
		includes raw bioassays that the logged in user doesn't have access to.
		@return The number of raw bioassays
	 	@throws BaseException If the raw bioassays couldn't be counted. 
	*/
	public long countRawBioAssays()
		throws BaseException
	{
		org.hibernate.Session session = getDbControl().getHibernateSession();
		org.hibernate.query.Query<Long> query = HibernateUtil.getPredefinedQuery(session, 
			"GET_RAWBIOASSAYS_FOR_DERIVEDBIOASSAY", Long.class, "count(*)");
		/*
			SELECT {1}
			FROM RawBioAssayData rba
			WHERE rba.parentBioAssay = :bioAssay
		*/
		query.setParameter("bioAssay", getId(), TypeWrapper.H_INTEGER);
		return HibernateUtil.loadData(query);
	}

	/**
		Whenever we modify the parent items to a derived bioassay,
		we must make sure that the physical bioassays for all
		child items are synchronized with the same changes.
		This is a bit complicated due to the fact that a single transaction
		may affect multiple derived bioassays at different levels in
		the parent-child tree. The fix must be made top-down and
		must handle accidentally created circular references.
		
		Whenever a change is deteced a single instance of this
		class is associated with the DbControl. All changed
		derived bioassays are collected with the {@link #add(DerivedBioAssayData)}
		method. The actual fix is done as a last step before committing
		the transaction {@link #onBeforeCommit()}.
	*/
	static class FixDerivedBioAssayParentsRecursivelyAction
		implements TransactionalAction
	{

		private static Map<DbControl, FixDerivedBioAssayParentsRecursivelyAction> cache = 
			Collections.synchronizedMap(new IdentityHashMap<DbControl, DerivedBioAssay.FixDerivedBioAssayParentsRecursivelyAction>());
		
		/**
			Get an instance tht handles the fixes for the given DbControl.
			If no instance exists yet, a new one is created.
		*/
		static FixDerivedBioAssayParentsRecursivelyAction get(DbControl dc)
		{
			FixDerivedBioAssayParentsRecursivelyAction dbt = cache.get(dc);
			if (dbt == null)
			{
				dbt = new FixDerivedBioAssayParentsRecursivelyAction(dc);
				cache.put(dc, dbt);
			}
			return dbt;
		}
		
		private final DbControl dc;
		private final Set<DerivedBioAssayData> allModified;
		
		private FixDerivedBioAssayParentsRecursivelyAction(DbControl dc)
		{
			this.dc = dc;
			this.allModified = new HashSet<DerivedBioAssayData>();
			dc.addTransactionalAction(this);
		}
		
		/**
			Register the given derived bioassay as one that has had it's
			parent items changed.
		*/
		void add(DerivedBioAssayData dba)
		{
			allModified.add(dba);
		}
		
		/*
		 	From the TransactionalAction interface
		 	--------------------------------------
		*/
		@Override
		public void onBeforeCommit() 
		{
			
			Set<DerivedBioAssayData> allToFix = new HashSet<DerivedBioAssayData>();
			List<DerivedBioAssayData> moreToFix = new LinkedList<DerivedBioAssayData>();
			
			// First step is to find all child items recursively
			// All non-root bioassays need to be fixed and all children may also have to be fixed
			for (DerivedBioAssayData dba : allModified)
			{
				if (!dba.isRoot()) allToFix.add(dba);
				if (dba.getChildren() != null)
				{
					moreToFix.addAll(dba.getChildren());
				}
			}
			
			// Check each child item and then also grandchildren
			while (moreToFix.size() > 0)
			{
				DerivedBioAssayData dba = moreToFix.remove(0);
				if (allToFix.add(dba) && dba.getChildren() != null)
				{
					moreToFix.addAll(dba.getChildren());
				}
			}
			// We now have all items that must be fixed...

			//System.out.println("allToFix: " + allToFix);

			// We don't know the actual order that items can be fixed, so
			// we have to keep trying until the set is empty
			while (allToFix.size() > 0)
			{
				int numFixed = 0;
				Iterator<DerivedBioAssayData> it = allToFix.iterator();
				while (it.hasNext())
				{
					DerivedBioAssayData dba = it.next();
					// Collect all physical bioassays from all parents in this set
					Set<PhysicalBioAssayData> allPhysicalBioAssays = new HashSet<PhysicalBioAssayData>();
					boolean canFix = true;
					
					// For each item that should be fixed, load all parent items...
					for (DerivedBioAssayData parent : dba.getParents())
					{
						if (allToFix.contains(parent))
						{
							// If at least one parent has not yet been fixed, we can't fix this item now
							canFix = false;
							break;
						}
						allPhysicalBioAssays.addAll(parent.getPhysicalBioAssays());
					}
					
					if (canFix)
					{
						//System.out.println("Fixing: " + dba.getName() + "; " + allPhysicalBioAssays);
						it.remove();
						dba.getPhysicalBioAssays().clear();
						dba.getPhysicalBioAssays().addAll(allPhysicalBioAssays);
						numFixed++;
					}
					else
					{
						//System.out.println("Can't fix " + dba.getName() + " yet, since it has unfixed parents");
					}
				}
				// If we couldn't fix any item in the last iteration, but there are more
				// items to fix, throw an exception since this is an indication of circular
				// reference
				if (numFixed == 0 && allToFix.size() > 0)
				{
					// Hmm... a circular reference prevents us from fixing 
					throw new InvalidDataException("Circular reference between derived bioassays: " + allToFix);
				}
			}
			
		}

		@Override
		public void onAfterCommit() 
		{
			cache.remove(dc);
		}

		@Override
		public void onRollback() 
		{
			cache.remove(dc);
		}
		// ---------------------------------------
		
	}
}
