/*
	$Id: ItemResultList.java 6875 2015-04-20 09:08:18Z nicklas $

	Copyright (C) 2005 Samuel Andersson, Nicklas Nordborg
	Copyright (C) 2006 Jari Häkkinen, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core;

import net.sf.basedb.core.data.BasicData;

import java.util.List;
import java.util.ListIterator;
import java.util.Iterator;
import java.lang.UnsupportedOperationException;

/**
	Return the results of an {@link ItemQuery} as a list. 
	
	@author Samuel, Nicklas
	@version 2.0
	@see ItemQuery#list(DbControl)
	@base.modified $Date: 2015-04-20 11:08:18 +0200 (Mon, 20 Apr 2015) $
*/
public class ItemResultList<I extends BasicItem>
	extends AbstractResultList<I>
{
	/**
		The internal list of BasicData objects.
	*/
	private final List<? extends BasicData> data;

	/**
		The DbControl that was used in the query.
	*/
	private final DbControl dc;

	/**
		The class of the items returned by the query (and this object). 
	*/
	private final Class<I> itemClass;
	
	/**
		The type of items returned by the query.
	*/
	private final Item itemType;

	ItemResultList(List<? extends BasicData> data, DbControl dc, Class<I> itemClass, long totalCount)
	{
		super(totalCount);
		assert data != null : "data == null";
		assert dc != null : "dc == null";
		assert itemClass != null : "itemClass == null";
		this.data = data;
		this.dc = dc;
		this.itemClass = itemClass;
		this.itemType = Item.fromItemClass(itemClass);
	}

	/*
		From the List interface
		-------------------------------------------
	*/
	@Override
	public boolean contains(Object o)
	{
		if (o instanceof BasicItem)
		{
			return data.contains(((BasicItem)o).getData());
		}
		else
		{
			return false;
		}
	}
	/**
		Get a BasicItem derived object.
		@return Object of type I or null if no valid item object
			could be fetched with the internal data object
	*/
	@Override
	public I get(int index)
	{
		return get(itemClass, data.get(index));
	}
	@Override
	public int indexOf(Object o)
	{
		if (o instanceof BasicItem)
		{
			return data.indexOf(((BasicItem)o).getData());
		}
		else
		{
			return -1;
		}
	}
	@Override
	public int lastIndexOf(Object o)
	{
		if (o instanceof BasicItem)
		{
			return data.lastIndexOf(((BasicItem)o).getData());
		}
		else
		{
			return -1;
		}
	}
	@Override
	public boolean isEmpty()
	{
		return data.isEmpty();
	}
	@Override
	public Iterator<I> iterator()
	{
		return new ResultListIterator<I>(data.iterator(), itemClass);
	}
	@Override
	public ListIterator<I> listIterator()
	{
		return new ResultListListIterator<I>(data.listIterator(), itemClass);
	}
	@Override
	public ListIterator<I> listIterator(int index)
	{
		return new ResultListListIterator<I>(data.listIterator(index), itemClass);
	}
	@Override
	public int size()
	{
		return data.size();
	}
	@Override
	public List<I> subList(int fromIndex, int toIndex)
	{
		return new ItemResultList<I>(data.subList(fromIndex, toIndex), dc, itemClass, getTotalCount());
	}
	@Override
	public Object[] toArray()
	{
		int size = size();
		Object[] items = new Object[size];
		for(int i = 0; i < size; ++i)
		{
			items[i] = get(i);
		}
		return items;
	}
	@Override
	@SuppressWarnings({"unchecked"})
	public <T> T[] toArray(T[] a)
	{
		int size = size();
		if(size > a.length)
		{
			a = (T[])java.lang.reflect.Array.newInstance(a.getClass().getComponentType(), size);
		}
		
		for(int i = 0; i < size; ++i)
		{
			a[i] = (T)get(i);
		}

		if(size < a.length)
		{
			a[size] = null;
		}
		return a;
	}
	// -------------------------------------------

	/*
		From the Object class
		-------------------------------------------
	*/
	@Override
	public boolean equals(Object o)
	{
		if (o instanceof ItemResultList)
		{
			return data.equals(((ItemResultList<?>)o).data);
		}
		else
		{
			return false;
		}
	}
	
	@Override
	public int hashCode()
	{
		return data.hashCode();
	}
	// -------------------------------------------

	/**
		Get the type of items contained in this list.
		@return An {@link Item} object or null if not known
	*/
	public Item getItemType()
	{
		return itemType;
	}
	
	private <E extends BasicItem> E get(Class<E> returnType, BasicData dataObject)
	{
		E itemObject = null;
		if (dataObject != null)
		{
			itemObject = dc.getItem(returnType, dataObject);
		}
		return itemObject;
	}

	/**
		Inner class to implement the iterator returned by
		the {@link #iterator()} method.
	*/
	private class ResultListIterator<T extends BasicItem>
		implements Iterator<T>
	{
		private final Iterator<? extends BasicData> iterator;
		private final Class<T> itemClass;

		private ResultListIterator(Iterator<? extends BasicData> iterator, Class<T> itemClass)
		{
			assert iterator != null : "iterator == null";
			assert itemClass != null : "itemClass == null";
			this.iterator = iterator;
			this.itemClass = itemClass;
		}

		/*
			From the Iterator interface
			-------------------------------------------
		*/
		@Override
		public boolean hasNext()
		{
			return iterator.hasNext();
		}
		@Override
		public T next()
		{
			return get(itemClass, iterator.next());
		}
		/**
			This operation is not supported.
			@throws UnsupportedOperationException Always
		*/
		@Override
		public void remove()
		{
			throw new UnsupportedOperationException();
		}
		// -------------------------------------------
	}

	/**
		Inner class to implement the iterator returned by
		the {@link #listIterator()} method.
	*/
	private class ResultListListIterator<T extends BasicItem>
		implements ListIterator<T>
	{
		private final ListIterator<? extends BasicData> iterator;
		private final Class<T> itemClass;

		private ResultListListIterator(ListIterator<? extends BasicData> iterator, Class<T> itemClass)
		{
			assert iterator != null : "iterator == null";
			assert itemClass != null : "itemClass == null";
			this.iterator = iterator;
			this.itemClass = itemClass;
		}

		/*
			From the ListIterator interface
			-------------------------------------------
		*/
		/**
			This operation is not supported.
			@throws UnsupportedOperationException Always
		*/
		@Override
		public void add(T o)
		{
			throw new UnsupportedOperationException();
		}
		@Override
		public boolean hasNext()
		{
			return iterator.hasNext();
		}
		@Override
		public boolean hasPrevious()
		{
			return iterator.hasPrevious();
		}
		@Override
		public T next()
		{
			return get(itemClass, iterator.next());
		}
		@Override
		public int nextIndex()
		{
			return iterator.nextIndex();
		}
		@Override
		public T previous()
		{
			return get(itemClass, iterator.previous());
		}
		@Override
		public int previousIndex()
		{
			return iterator.previousIndex();
		}
		/**
			This operation is not supported.
			@throws UnsupportedOperationException Always
		*/
		@Override
		public void remove()
		{
			throw new UnsupportedOperationException();
		}
		/**
			This operation is not supported.
			@throws UnsupportedOperationException Always
		*/
		@Override
		public void set(T o)
		{
			throw new UnsupportedOperationException();
		}
		// -------------------------------------------
	}
}
