/*
	$Id $

	Copyright (C) 2017 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core;

import net.sf.basedb.core.data.UserDeviceData;
import net.sf.basedb.core.query.Restrictions;
import net.sf.basedb.core.query.EntityQuery;
import net.sf.basedb.core.query.Hql;

import java.util.Date;

/**
	A user device represents a verified device a user may
	use for accessing BASE.

	@author Nicklas
	@since 3.12
*/
public class UserDevice
	extends BasicItem
	implements Registered, Nameable
{
	/**
		The type of item represented by this class.
		@see Item#USERDEVICE
		@see #getType()
	*/
	public static final Item TYPE = Item.USERDEVICE;

	/**
		This filter will limit a query to only return devices
		for the logged in user unless the logged in user has generic 
		read permission.
	*/
	private static final QueryRuntimeFilter RUNTIME_FILTER = new QueryRuntimeFilterImpl();

	/**
		Get a <code>UserDevice</code> item when you know the ID.

		@param dc The <code>DbControl</code> which will be used for
			permission checking and database access.
		@param id The ID of the item to load
		@return The <code>UserDevice</code> item
	*/
	public static UserDevice getById(DbControl dc, int id)
		throws ItemNotFoundException, PermissionDeniedException, BaseException
	{
		UserDevice dev = dc.loadItem(UserDevice.class, id);
		if (dev == null) throw new ItemNotFoundException("UserDevice[id="+id+"]");
		return dev;
	}

	/**
		Get a query configured to retrieve devices for the specified user.
		
		@param user The user to retreive devices for, null is allowed if
			the logged in user has generic READ permission for devices in which 
			case all devices will be returned
		@return An {@link ItemQuery} object
	*/
	public static ItemQuery<UserDevice> getQuery(User user)
	{
		ItemQuery<UserDevice> query = new ItemQuery<UserDevice>(UserDevice.class, RUNTIME_FILTER);
		if (user != null)
		{
			query.include(Include.OTHERS);
			query.restrictPermanent(
				Restrictions.eq(
					Hql.property("user"), 
					Hql.entity(user)
				)
			);
		}
		return query;
	}

	UserDevice(UserDeviceData deviceData)
	{
		super(deviceData);
	}

	@Override
	UserDeviceData getData()
	{
		return (UserDeviceData)getBasicData();
	}
	/*
		From the Identifiable interface
		-------------------------------------------
	*/
	@Override
	public Item getType()
	{
		return TYPE;
	}
	// -------------------------------------------

	/*
		From the BasicItem class
		-------------------------------------------
	*/
	/**
		DELETE permission is granted if the logged in user is the owner of this
		device.
	*/
	@Override
	void initPermissions(int granted, int denied)
		throws BaseException
	{
		if (getSessionControl().getLoggedInUserId() == getData().getUser().getId())
		{
			granted |= Permission.grant(Permission.DELETE);
		}
		super.initPermissions(granted, denied);
	}
	// -------------------------------------------
	/*
		From the Registered interface
		-------------------------------------------
	*/
	@Override
	public Date getEntryDate()
	{
		return DateUtil.copy(getData().getEntryDate());
	}
	@Override
	public void setEntryDate(Date entryDate)
	{
		if (isInDatabase()) 
		{
			throw new PermissionDeniedException("Not allowed to change entryDate of " + this);
		}
		getData().setEntryDate(DateUtil.setDateOrToday(entryDate));
	}
	// -------------------------------------------
	/*
		From the Nameable interface
		-------------------------------------------
	*/
	@Override
	public String getName()
	{
		return getData().getName();
	}
	@Override
	public void setName(String name)
		throws PermissionDeniedException, InvalidDataException
	{
		checkPermission(Permission.WRITE);
		NameableUtil.setName(getData(), name);
	}
	@Override
	public String getDescription()
	{
		return getData().getDescription();
	}
	@Override
	public void setDescription(String description)
		throws PermissionDeniedException, InvalidDataException
	{
		checkPermission(Permission.WRITE);
		NameableUtil.setDescription(getData(), description);
	}
	// -------------------------------------------

	/**
		Get the user that owns this device.

		@return The <code>User</code> item
		@throws PermissionDeniedException If the logged in user doesn't have 
			{@link Permission#READ} permission
		@throws BaseException If there is another error
	*/
	public User getUser()
		throws PermissionDeniedException, BaseException
	{
		return getDbControl().getItem(User.class, getData().getUser());
	}

	/**
		Get the client application the user was using with this device.

		@return A <code>Client</code> item, or null if not known
		@throws PermissionDeniedException If the logged in user doesn't 
			have {@link Permission#READ} permission
		@throws BaseException If there is another error
	*/
	public Client getClient()
		throws PermissionDeniedException, BaseException
	{
		return getDbControl().getItem(Client.class, getData().getClient());
	}

	/**
		Get the date and time the user last used this device.
		@return A <code>Date</code> object with the date and time
	*/
	public Date getLastUsed()
	{
		return DateUtil.copy(getData().getLastUsed());
	}

	/**
		Get the user agent string of this device.
	*/
	public String getUserAgent()
	{
		return getData().getUserAgent();
	}
	
	/**
		Get the remote ID of the host the user used for this device the last time.
		Typically it is the IP-address of the user's computer.
		@return A <code>String</code> object with remote ID
	*/
	public String getLastRemoteId()
	{
		return getData().getLastRemoteId();
	}
	
	/**
		Get the last known location of the device.
		@return A <code>String</code> object with location or null if not known
	*/
	public String getLocation()
	{
		return getData().getLocation();
	}
	
	/**
		Get the last known location latitude of the device.
		@return A <code>Float</code> object with latitude or null if not known
	*/
	public Float getLocationLatitude()
	{
		return getData().getLocationLatitude();
	}

	/**
		Get the last known location longitude of the device.
		@return A <code>Float</code> object with longitude or null if not known
	*/
	public Float getLocationLongitude()
	{
		return getData().getLocationLongitude();
	}
	
	/**
		Has this device been verified or not? Verification is done
		by requesting a code via email.
		@since 3.17
	*/
	public boolean isVerified()
	{
		return getData().isVerified();
	}

	/**
		Get a query for loading login session from this device.
		@since 3.17
	*/
	public ItemQuery<Session> getSessions()
	{
		ItemQuery<Session> query = Session.getQuery(getUser());
		query.restrictPermanent(Restrictions.eq(Hql.property("device"), Hql.entity(this)));
		return query;
	}
	
	/**
		A runtime filter implementation that limits a query to only
		return devices for the currently logged in user unless the logged 
		in user has generic read permission.
	*/
	private static class QueryRuntimeFilterImpl
		implements QueryRuntimeFilter
	{
		@Override
		public void enableFilters(QueryRuntimeFilterManager manager, EntityQuery query, DbControl dc)
		{
			SessionControl sc = dc.getSessionControl();
			if (sc.isDenied(query.getItemPermission(), query.getItemType()) || !sc.isLoggedIn())
			{
				manager.enableFilter("denyAll");
			}
			else if (query.isIncluded(Include.OTHERS) && 
				sc.hasPermission(query.getItemPermission(), query.getItemType()))
			{
				// Logged in user has generic permission and wants other user's items... --> no filter
			}
			else
			{
				// Load sessions for the logged in user
				org.hibernate.Filter filter = manager.enableFilter("ownedBy");
				if (filter != null)
				{
					filter.setParameter("owner", sc.getLoggedInUserId());
				}
			}
		}
	}

}



