/**
	$Id $

	Copyright (C) 2014 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core.authentication;

import java.util.HashMap;
import java.util.Map;

import net.sf.basedb.core.SessionControl;

/**
	Hold information about a login request such as login, password, etc.
	
	@author Nicklas
	@since 3.3
	@base.modified $Date $
*/
public class LoginRequest
{
	private boolean verifyOnly;
	private int userId;
	private String login;
	private String password;
	private String deviceToken;
	private String comment;
	
	private Map<String, String> attributes;
	
	public LoginRequest()
	{}
	
	/**
		Create a login request with login + password
	*/
	public LoginRequest(String login, String password)
	{
		this.login = login;
		this.password = password;
	}
	
	/**
		Create a login request with login + password + deviceToken
		@since 3.12
	*/
	public LoginRequest(String login, String password, String deviceToken)
	{
		this.login = login;
		this.password = password;
		this.deviceToken = deviceToken;
	}
	
	/**
		Create a login request with user id + password
	*/
	public LoginRequest(int userId, String password)
	{
		this.userId = userId;
		this.password = password;
	}

	/**
		Create a login request with user id + password + deviceToken
		@since 3.12
	*/
	public LoginRequest(int userId, String password, String deviceToken)
	{
		this.userId = userId;
		this.password = password;
		this.deviceToken = deviceToken;
	}

	/**
		Set a flag to indicate if the login request should only verify
		the login parameters and not do a full login.
		@since 3.19.3
	*/
	public void setVerifyOnly(boolean verifyOnly)
	{
		this.verifyOnly = verifyOnly;
	}
	
	/**
		If this flag is set, the login parameters are only verified and
		the actual login is aborted. If the verification is successful, the
		{@link SessionControl#login(LoginRequest)} returns normally, otherwise
		an exception is thrown.
		@since 3.19.3
	*/
	public boolean isVerifyOnly()
	{
		return verifyOnly;
	}
	
	/**
		Set the user id to use.
	*/
	public void setUserId(int userId)
	{
		this.userId = userId;
	}
	
	/**
		Get the user id to use.
	*/
	public int getUserId()
	{
		return userId;
	}
	
	/**
		Set the login name to use.
	*/
	public void setLogin(String login)
	{
		this.login = login;
	}
	
	/**
		Get the login name to use.
	*/
	public String getLogin()
	{
		return login;
	}
	
	/**
		Set the password to use.
	*/
	public void setPassword(String password)
	{
		this.password = password;
	}
	
	/**
		Get the password to use.
	*/
	public String getPassword()
	{
		return password;
	}
	
	/**
		Set the deviceToken to use.
		@since 3.12
	*/
	public void setDeviceToken(String deviceToken)
	{
		this.deviceToken = deviceToken;
	}
	
	/**
		Get the deviceToken to use.
		@since 3.12
	*/
	public String getDeviceToken()
	{
		return deviceToken;
	}

	/**
		Set the comment to use.
	*/
	public void setComment(String comment)
	{
		this.comment = comment;
	}
	
	/**
		Get the comment to use.
	*/
	public String getComment()
	{
		return comment;
	}
	
	/**
		Set an extra attribute to use for the login request.
	*/
	public void setAttribute(String key, String value)
	{
		if (attributes == null) attributes = new HashMap<String, String>();
		attributes.put(key, value);
	}
	
	/**
		Get the value of an extra attribute to use for the login request.
	*/
	public String getAttribute(String key)
	{
		return attributes == null ? null : attributes.get(key);
	}

}

