/**
	$Id: ChangeHistoryData.java 7946 2021-05-07 11:30:22Z nicklas $

	Copyright (C) 2009 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core.data;

import java.util.Date;
import java.util.Set;

/**
	Represents the changes made in a single transaction. This
	entity records information about the time, user, active
	project, used plug-in, etc. Changes made to entities in
	the transaction are recorded with {@link ChangeHistoryDetailData}
	objects.
	<p>
	NOTE! All references to other items are controlled by regular integer 
	values because we don't want foreign keys from the log tables to block 
	deletion of items. A duplicate mapping using many-to-one with a formula
	attribute is used since we really need the LEFT JOIN functionality in the
	web interface and the association is also useful for data export which 
	uses metadata to access the properties. The formula mapping is not
	supported by XDoclet so we have to do this in the 
	hibernate-properties.ChangeHistoryData.xml file.
	<p>
	Missing items are handled by the core layer at runtime. 
	References in the log to missing items may optionally be deleted by a 
	cleanup thread from time to time.
	<p>
	
	
	@author Nicklas
	@version 2.13
	@base.modified $Date: 2021-05-07 13:30:22 +0200 (Fri, 07 May 2021) $
	@hibernate.class table="`ChangeHistory`" lazy="true" mutable="false"
	@see <a href="../../../../../../html/developer/api/data_api.misc.html">Developer documentation: Other classes</a>
*/
public class ChangeHistoryData
	extends BasicData
{
	
	public ChangeHistoryData()
	{}
	
	private Date time;
	/**
		Get the time of the change.
		@return A <code>Date</code> object
		@hibernate.property column="`time`" type="timestamp" not-null="true"
	*/
	public Date getTime()
	{
		return time;
	}
	public void setTime(Date time)
	{
		this.time = time;
	}

	private String name;
	/**
		Get the name of the transaction.
		@return A <code>String</code> with the name of the transaction
		@since 3.19
		@hibernate.property type="string"
		@hibernate.column name="`name`" length="255" not-null="true" index="name_idx"
	*/
	public String getName()
	{
		return name;
	}
	public void setName(String name)
	{
		this.name = name;
	}
	
	private int userId;
	/**
		The ID of the user that made the change.
		@hibernate.property type="int"
		@hibernate.column name="`user_id`" not-null="true"
	*/
	public int getUserId()
	{
		return userId;
	}
	public void setUserId(int userId)
	{
		this.userId = userId;
	}

	private UserData user;
	UserData getUser()
	{
		return user;
	}
	void setUser(UserData user)
	{
		this.user = user;
	}
	
	private int sessionId;
	/**
		The ID of the session in which the changes happened.
		@hibernate.property type="int"
		@hibernate.column name="`session_id`" not-null="true"
	*/
	public int getSessionId()
	{
		return sessionId;
	}
	public void setSessionId(int sessionId)
	{
		this.sessionId = sessionId;
	}

	private SessionData session;
	SessionData getSession()
	{
		return session;
	}
	void setSession(SessionData session)
	{
		this.session = session;
	}
	
	private Integer clientId;
	/**
		The ID of the client application the user used to make the change.
		@hibernate.property type="int"
		@hibernate.column name="`client_id`" not-null="false"
	*/
	public Integer getClientId()
	{
		return clientId;
	}
	public void setClientId(Integer clientId)
	{
		this.clientId = clientId;
	}

	private ClientData client;
	ClientData getClient()
	{
		return client;
	}
	void setClient(ClientData client)
	{
		this.client = client;
	}

	
	private Integer projectId;
	/**
		The ID of the project that was active when the changes were made.
		@hibernate.property type="int"
		@hibernate.column name="`project_id`" not-null="false"
	*/
	public Integer getProjectId()
	{
		return projectId;
	}
	public void setProjectId(Integer projectId)
	{
		this.projectId = projectId;
	}
	
	private ProjectData project;
	ProjectData getProject()
	{
		return project;
	}
	void setProject(ProjectData project)
	{
		this.project = project;
	}
	
	private Integer pluginId;
	/**
		The ID of the plug-in that was making the changes.
		@hibernate.property type="int"
		@hibernate.column name="`plugin_id`" not-null="false"
	*/
	public Integer getPluginId()
	{
		return pluginId;
	}
	public void setPluginId(Integer pluginId)
	{
		this.pluginId = pluginId;
	}

	private PluginDefinitionData plugin;
	PluginDefinitionData getPlugin()
	{
		return plugin;
	}
	void setPlugin(PluginDefinitionData plugin)
	{
		this.plugin = plugin;
	}
	
	private Integer jobId;
	/**
		The ID of the job that was making the changes.
		@hibernate.property type="int"
		@hibernate.column name="`job_id`" not-null="false"
	*/
	public Integer getJobId()
	{
		return jobId;
	}
	public void setJobId(Integer jobId)
	{
		this.jobId = jobId;
	}
	
	private JobData job;
	JobData getJob()
	{
		return job;
	}
	void setJob(JobData job)
	{
		this.job = job;
	}
	
	private Set<ChangeHistoryDetailData> details;
	/**
		This is the inverse end.
		@see ChangeHistoryDetailData#getChangeHistory()
		@hibernate.set lazy="true" inverse="true" cascade="delete"
		@hibernate.collection-key column="`history_id`"
		@hibernate.collection-one-to-many class="net.sf.basedb.core.data.ChangeHistoryDetailData"
	*/
	Set<ChangeHistoryDetailData> getDetails()
	{
		return details;
	}
	void setDetails(Set<ChangeHistoryDetailData> details)
	{
		this.details = details;
	}
	
}
