/*
	$Id: JobData.java 7437 2018-01-17 12:44:16Z nicklas $

	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Häkkinen, Nicklas Nordborg
	Copyright (C) 2007 Nicklas Nordborg, Martin Svensson

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core.data;

import java.util.Date;
import java.util.Set;
import java.util.Map;
import java.util.HashMap;

/**
	This class holds information about a job.

	@author Nicklas
	@version 2.0
	@see net.sf.basedb.core.Job
	@see <a href="../../../../../../html/developer/api/data_api.plugins.html">Developer documentation: Plug-ins, jobs and job agents</a>
	@base.modified $Date: 2018-01-17 13:44:16 +0100 (Wed, 17 Jan 2018) $
	@hibernate.class table="`Jobs`" lazy="true"
*/
public class JobData
	extends CommonData
	implements SubtypableData
{

	public JobData()
	{}

	private int type;
	/**
		The type of job: 1 = run plugin, 2 = other
		@hibernate.property column="`type`" type="int" not-null="true"
	*/
	public int getType()
	{
		return type;
	}
	public void setType(int type)
	{
		this.type = type;
	}

	/*
		From the SubtypableData interface
		----------------------------------
	*/
	private ItemSubtypeData subtype;
	/**
		@since 3.10
	*/
	@Override
	public ItemSubtypeData getItemSubtype()
	{
	   return subtype;
	}
	@Override
	public void setItemSubtype(ItemSubtypeData subtype)
	{
	   this.subtype = subtype;
	}
	// ------------------------------

	/**
		The maximum allowed length of the external ID.
		@since 3.3
	*/
	public static final int MAX_EXTERNAL_ID_LENGTH = 255;
	private String externalId;
	/**
		External ID of a job running on an external system.
		@hibernate.property column="`external_id`" type="string" length="255" not-null="false"
		@since 3.3
	*/
	public String getExternalId()
	{
		return externalId;
	}
	public void setExternalId(String externalId)
	{
		this.externalId = externalId;
	}
	
	private PluginDefinitionData pluginDefinition;
	/**
		The plugin that executes the job.
		@hibernate.many-to-one column="`plugindefinition_id`" not-null="false" outer-join="false" update="false"
	*/
	public PluginDefinitionData getPluginDefinition()
	{
		return pluginDefinition;
	}
	public void setPluginDefinition(PluginDefinitionData pluginDefinition)
	{
		this.pluginDefinition = pluginDefinition;
	}

	
	private PluginConfigurationData pluginConfiguration;
	/**
		The plugin configuration for the plugin.
		@hibernate.many-to-one column="`pluginconfiguration_id`" not-null="false" outer-join="false" update="false"
	*/
	public PluginConfigurationData getPluginConfiguration()
	{
		return pluginConfiguration;
	}
	public void setPluginConfiguration(PluginConfigurationData pluginConfiguration)
	{
		this.pluginConfiguration = pluginConfiguration;
	}

	private ExperimentData experiment;
	
	/**
		The experiment this job is a part of.
		@since 2.4
		@hibernate.many-to-one column="`experiment_id`" not-null="false" outer-join="false"
	*/
	public ExperimentData getExperiment()
	{
		return experiment;
	}
	public void setExperiment(ExperimentData experiment)
	{
		this.experiment = experiment;
	}
	
	/**
		The maximum allowed length of the plugin version.
		@since 2.8
	*/
	public static final int MAX_PLUGIN_VERSION_LENGTH = PluginDefinitionData.MAX_VERSION_LENGTH;
	private String pluginVersion;
	/**
		Get the version of the plugin used to execute this job at the time the
		job was executed or registered.
		@return A string with the version of the plugin, or null if not known
			or if the job is not executed by a plug-in
		@hibernate.property column="`version_string`" type="string" length="255" not-null="false"
		@since 2.8
	*/
	public String getPluginVersion()
	{
		return pluginVersion;
	}
	/**
		@since 2.8
	*/
	public void setPluginVersion(String pluginVersion)
	{
		this.pluginVersion = pluginVersion;
	}
	
	private int parameterVersion;
	/**
		Get the parameter version of the configuration  used for this job.
		@hibernate.property column="`parameter_version`" type="int" not-null="true"
	*/
	public int getParameterVersion()
	{
		return parameterVersion;
	}
	public void setParameterVersion(int parameterVersion)
	{
		this.parameterVersion = parameterVersion;
	}
	
	private boolean sendMessage;
	/**
		If the core should send a message to the owner when the job is finished
		or not.
		@hibernate.property column="`send_message`" type="boolean" not-null="true"
	*/
	public boolean getSendMessage()
	{
		return sendMessage;
	}
	public void setSendMessage(boolean sendMessage)
	{
		this.sendMessage = sendMessage;
	}
	
	private boolean removeJobWhenFinished;
	/**
		If the core should delete the job when it's finished successfully.
		@hibernate.property column="`remove_job`" type="boolean" not-null="true"
	*/
	public boolean getRemoveJobWhenFinished()
	{
		return removeJobWhenFinished;
	}
	public void setRemoveJobWhenFinished(boolean removeJobWhenFinished)
	{
		this.removeJobWhenFinished = removeJobWhenFinished;
	}
	
	private int status;
	/**
		Get the status of the job. 1 = waiting, 2 = running, 3 = completed ok, 4 = error
		@hibernate.property column="`status`" type="int" not-null="true"
	*/
	public int getStatus()
	{
		return status;
	}
	public void setStatus(int status)
	{
		this.status = status;
	}

	private boolean dryRun;
	/**
		If the job should be/was executed as a dry-run job.
		@hibernate.property column="`dry_run`" type="boolean" not-null="true"
	*/
	public boolean isDryRun()
	{
		return dryRun;
	}
	public void setDryRun(boolean dryRun)
	{
		this.dryRun = dryRun;
	}
	
	/**
		The maximum allowed length of the execute command.
		@since 2.16
	*/
	public static final int MAX_COMMAND_LENGTH = 255;
	private String executeCommand;
	/**
		The command to send to the plug-in when starting to
		execute it. If null, a default values is used. See
		Request.COMMAND_EXECUTE. This is usually only used when
		automatically restarting a failed job.
		@hibernate.property column="`exec_command`" type="string" length="255" not-null="false"
		@since 2.16
	*/
	public String getExecuteCommand()
	{
		return executeCommand;
	}
	public void setExecuteCommand(String command)
	{
		this.executeCommand = command;
	}
	
	/**
		The maximum allowed length of the status message.
	*/
	public static final int MAX_STATUS_MESSAGE_LENGTH = 65535;
	private String statusMessage;
	/**
		Get a status message.
		@hibernate.property column="`status_message`" type="text" not-null="false"
	*/
	public String getStatusMessage()
	{
		return statusMessage;
	}
	public void setStatusMessage(String statusMessage)
	{
		this.statusMessage = statusMessage;
	}
	
	/**
		The maximum allowed length of the stack trace message.
	*/
	public static final int MAX_STACK_TRACE_LENGTH = 65535;
	private String stackTrace;
	/**
		Get the stack trace in case of an error
		@hibernate.property column="`stack_trace`" type="text" not-null="false"
	*/
	public String getStackTrace()
	{
		return stackTrace;
	}
	public void setStackTrace(String stackTrace)
	{
		this.stackTrace = stackTrace;
	}
	
	private int estimatedExecutionTime;
	/**
		Get the estimated execution time of the job. 
		0 = &lt;1 minute, 1 = 1 - 10 minutes, 2 =  &lt;1 hour, 3 = &gt;1 hour
		@hibernate.property column="`execution_time`" type="int" not-null="true"
	*/
	public int getEstimatedExecutionTime()
	{
		return estimatedExecutionTime;
	}
	public void setEstimatedExecutionTime(int estimatedExecutionTime)
	{
		this.estimatedExecutionTime = estimatedExecutionTime;
	}

	private int percentComplete;
	/**
		If the job is running, how many percent has been completed.
		@hibernate.property column="`percent_complete`" type="int" not-null="true"
	*/
	public int getPercentComplete()
	{
		return percentComplete;
	}
	public void setPercentComplete(int percentComplete)
	{
		this.percentComplete = percentComplete;
	}

	private int priority;
	/**
		The jobs priority. A lower value means a higher priority.
		@hibernate.property column="`priority`" type="int" not-null="true"
	*/
	public int getPriority()
	{
		return priority;
	}
	public void setPriority(int priority)
	{
		this.priority = priority;
	}

	private int activeProjectId;
	/**
		The ID of the project that should be made active when running this job.
		@hibernate.property column="`project_id`" type="int" not-null="true"
	*/
	public int getActiveProjectId()
	{
		return activeProjectId;
	}
	public void setActiveProjectId(int activeProjectId)
	{
		this.activeProjectId = activeProjectId;
	}
	
	private Date created;
	/**
		Get the date and time the job was created.
		@hibernate.property column="`created`" type="timestamp" not-null="true" update="false"
	*/
	public Date getCreated()
	{
		return created;
	}
	public void setCreated(Date created)
	{
		this.created = created;
	}

	private Date scheduled;
	/**
		Get the date and time the job was scheduled to the job queue, or null
		if it hasn't been scheduled yet. This timestamp should be updated whenever
		the status is changed to WAITING.
		@hibernate.property column="`scheduled`" type="timestamp" not-null="false"
		@since 2.16
	*/
	public Date getScheduled()
	{
		return scheduled;
	}
	public void setScheduled(Date scheduled)
	{
		this.scheduled = scheduled;
	}
	
	private Date started;
	/**
		Get the date and time the job was started or null if the job hasn't been started.
		@hibernate.property column="`started`" type="timestamp" not-null="false"
	*/
	public Date getStarted()
	{
		return started;
	}
	public void setStarted(Date started)
	{
		this.started = started;
	}
	
	private Date ended;
	/**
		Get the date and time the job was ended or null if the job hasn't ended.
		@hibernate.property column="`ended`" type="timestamp" not-null="false"
	*/
	public Date getEnded()
	{
		return ended;
	}
	public void setEnded(Date ended)
	{
		this.ended = ended;
	}
	
	/**
		The maximum allowed length of the server name.
	*/
	public static final int MAX_SERVER_LENGTH = 255;
	private String server;
	/**
		The name of the server where the job is executing.
		@hibernate.property column="`server`" type="string" length="255" not-null="false"
	*/
	public String getServer()
	{
		return server;
	}
	public void setServer(String server)
	{
		this.server = server;
	}
	
	private String node;
	/**
		The name of the node where the job is executing. If this value is used
		it typically means that the server is some kind of master controller
		for multiple nodes in a cluster. The max length is the same
		as for the server.
		@since 3.10
		@hibernate.property column="`node`" type="string" length="255" not-null="false"
	*/
	public String getNode()
	{
		return node;
	}
	public void setNode(String node)
	{
		this.node = node;
	}

	private Integer contextItem;
	/**
		The item type of the context that was active when configuring
		this job. null = no context was known
		@hibernate.property column="`context_item`" type="int"
		@since 2.8
	*/
	public Integer getContextItem()
	{
		return contextItem;
	}
	/**
		@since 2.8
	*/
	public void setContextItem(Integer contextItem)
	{
		this.contextItem = contextItem;
	}
	
	private Integer contextType;
	/**
		The context type that was active when configuring
		this job. 0 = single item, 1 = list
		@hibernate.property column="`context_type`" type="int"
		@since 2.8
	*/
	public Integer getContextType()
	{
		return contextType;
	}
	/**
		@since 2.8
	*/
	public void setContextType(Integer contextType)
	{
		this.contextType = contextType;
	}
	
	/**
		The maximum allowed length of the subcontext name.
		@since 2.8
	*/
	public static final int MAX_SUBCONTEXT_LENGTH = 255;
	private String subContext;
	/**
		The name of the subcontext that was active whjen configuring
		this job.
		@hibernate.property column="`subcontext`" type="string" length="255" not-null="false"
		@since 2.8
	*/
	public String getSubContext()
	{
		return subContext;
	}
	/**
		@since 2.8
	*/
	public void setSubContext(String subContext)
	{
		this.subContext = subContext;
	}
	
	/**
		The maximum allowed length of the signal transporter string.
	*/
	public static final int MAX_SIGNAL_TRANSPORTER_LENGTH = 655536;
	private String signalTransporter;
	/**
		Identifies the signal transporter to use. This is a string with 
		two parts separated by colon (:). The first part is the class name
		of the signal transporter class, the second part is the initialisation
		string for the transporter.
		@hibernate.property column="`signal_transporter`" type="text" not-null="false"
		@since 2.6
	*/
	public String getSignalTransporter()
	{
		return signalTransporter;
	}
	public void setSignalTransporter(String signalTransporter)
	{
		this.signalTransporter = signalTransporter;
	}
	
	private Integer jobAgentId;
	/**
		The ID of the job agent this job should run/is running/was running on.
		We store this as an ID value only and not a reference to {@link JobAgentData}
		since it would then be impossible to delete a job agent without also deleting
		all jobs that has been running on it.
		@since 2.8
		@hibernate.property column="`jobagent_id`" type="int" not-null="false"
	*/
	public Integer getJobAgentId()
	{
		return jobAgentId;
	}
	public void setJobAgentId(Integer jobAgentId)
	{
		this.jobAgentId = jobAgentId;
	}
	
	private Map<String, ParameterValueData<?>> parameters;
	/**
		The parameters for this job.
		@hibernate.map table="`JobParameters`" lazy="true" cascade="all"
		@hibernate.collection-key column="`job_id`"
		@hibernate.collection-index column="`name`" type="string" length="255"
		@hibernate.collection-many-to-many column="`value_id`" class="net.sf.basedb.core.data.ParameterValueData"
	*/
	public Map<String, ParameterValueData<?>> getParameters()
	{
		if (parameters == null) parameters = new HashMap<String, ParameterValueData<?>>();
		return parameters;
	}
	void setParameters(Map<String, ParameterValueData<?>> parameters)
	{
		this.parameters = parameters;
	}

	private Set<MessageData> messages;
	/**
		This is the inverse end.
		@see MessageData#getJob()
		@hibernate.set lazy="true" inverse="true" cascade="delete"
		@hibernate.collection-key column="`job_id`"
		@hibernate.collection-one-to-many class="net.sf.basedb.core.data.MessageData"
	*/
	Set<MessageData> getMessages()
	{
		return messages;
	}
	void setMessages(Set<MessageData> messages)
	{
		this.messages = messages;
	}

	private Set<DerivedBioAssayData> derivedBioAssays;
	/**
		This is the inverse end.
		@see DerivedBioAssayData#getJob()
		@hibernate.set lazy="true" inverse="true" 
		@hibernate.collection-key column="`job_id`"
		@hibernate.collection-one-to-many class="net.sf.basedb.core.data.DerivedBioAssayData"
		@since 3.3
	*/
	Set<DerivedBioAssayData> getDerivedBioAssays()
	{
		return derivedBioAssays;
	}
	void setDerivedBioAssays(Set<DerivedBioAssayData> derivedBioAssays)
	{
		this.derivedBioAssays = derivedBioAssays;
	}

	private Set<RawBioAssayData> rawBioAssays;
	/**
		This is the inverse end.
		@see RawBioAssayData#getJob()
		@hibernate.set lazy="true" inverse="true" 
		@hibernate.collection-key column="`job_id`"
		@hibernate.collection-one-to-many class="net.sf.basedb.core.data.RawBioAssayData"
		@since 3.3
	*/
	Set<RawBioAssayData> getRawBioAssays()
	{
		return rawBioAssays;
	}
	void setRawBioAssays(Set<RawBioAssayData> rawBioAssays)
	{
		this.rawBioAssays = rawBioAssays;
	}

	private Set<ArrayDesignData> arrayDesigns;
	/**
		This is the inverse end.
		@see ArrayDesignData#getJob()
		@hibernate.set lazy="true" inverse="true" 
		@hibernate.collection-key column="`job_id`"
		@hibernate.collection-one-to-many class="net.sf.basedb.core.data.ArrayDesignData"
		@since 3.3
	*/
	Set<ArrayDesignData> getArrayDesigns()
	{
		return arrayDesigns;
	}
	void setArrayDesigns(Set<ArrayDesignData> arrayDesigns)
	{
		this.arrayDesigns = arrayDesigns;
	}

	private Set<TransformationData> transformations;
	/**
		This is the inverse end.
		@see TransformationData#getJob()
		@hibernate.set lazy="true" inverse="true" 
		@hibernate.collection-key column="`job_id`"
		@hibernate.collection-one-to-many class="net.sf.basedb.core.data.TransformationData"
		@since 3.3
	*/
	Set<TransformationData> getTransformations()
	{
		return transformations;
	}
	void setTransformations(Set<TransformationData> transformations)
	{
		this.transformations = transformations;
	}

}
