/*
 $Id: SessionData.java 7414 2017-10-12 12:07:09Z nicklas $

	Copyright (C) 2005 Johan Enell, Jari Häkkinen, Nicklas Nordborg
	Copyright (C) 2006 Jari Häkkinen, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

 BASE is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 3
 of the License, or (at your option) any later version.

 BASE is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with BASE. If not, see <http://www.gnu.org/licenses/>.
 */
package net.sf.basedb.core.data;

import java.util.Date;

/**
	This class holds information about a session.
 
	@author enell
	@version 2.0
	@see net.sf.basedb.core.Session
	@see <a href="../../../../../../html/developer/api/data_api.clients.html">Developer documentation: Client, session and settings</a>
	@hibernate.class table="`Sessions`" lazy="false"
*/
public class SessionData
	extends BasicData
{
	public SessionData()
	{}
	
	private UserData user;
	/**
		Get the user that logged in with this session.
		@hibernate.many-to-one column="`user_id`" not-null="true" update="false" outer-join="false"
	*/
	public UserData getUser()
	{
		return user;
	}
	
	public void setUser(UserData user)
	{
		this.user = user;
	}
		
	private Date loginTime;
	/**
		Get the date and time the user logged in.
		@hibernate.property column="`login_time`" type="timestamp" not-null="true" update="false"
	*/
	public Date getLoginTime()
	{
		return loginTime;
	}
	
	public void setLoginTime(Date loginTime)
	{
		this.loginTime = loginTime;
	}

	private Date logoutTime;
	/**
		Get the date and time the user logged out.
		@hibernate.property column="`logout_time`" type="timestamp" not-null="false"
	*/
	public Date getLogoutTime()
	{
		return logoutTime;
	}
	
	public void setLogoutTime(Date logoutTime)
	{
		this.logoutTime = logoutTime;
	}

	/**
		The maximum length of the login comment that can be stored in the database.
		@see #setLoginComment(String)
	*/
	public static final int MAX_LOGIN_COMMENT_LENGTH = 65535;
	private String loginComment;
	/**
		Get the login comment.
		@hibernate.property column="`login_comment`" type="text" not-null="false" update="false"
	*/
	public String getLoginComment()
	{
		return loginComment;
	}
	
	public void setLoginComment(String loginComment)
	{
		this.loginComment = loginComment;
	}

	private boolean impersonated = false;
	/**
		Check if another user is acting as the user of this session.
		@hibernate.property column="`impersonated`" type="boolean" not-null="true" update="false"
	*/
	public boolean getImpersonated()
	{
		return impersonated;
	}
	
	public void setImpersonated(boolean impersonated)
	{
		this.impersonated = impersonated;
	}

	private ClientData client;
	/**
		Get the client application the user was using.
		@hibernate.many-to-one column="`client_id`" not-null="false" update="false" outer-join="false"
	*/
	public ClientData getClient()
	{
		return client;
	}
	
	public void setClient(ClientData client)
	{
		this.client = client;
	}
	
	private UserDeviceData device;
	/**
		Get the device the user was using.
		@since 3.12
		@hibernate.many-to-one column="`device_id`" not-null="false" update="false" outer-join="false"
	*/
	public UserDeviceData getDevice()
	{
		return device;
	}
	
	public void setDevice(UserDeviceData device)
	{
		this.device = device;
	}

	
	/**
		The maximum length of the remote ID that can be stored in the database.
		@see #setRemoteId(String)
	*/
	public static final int MAX_REMOTE_ID_LENGTH = 255;
	private String remoteId;
	/**
		Get the login comment.
		@hibernate.property column="`remote_id`" type="string" length="255" not-null="false" update="false"
	*/
	public String getRemoteId()
	{
		return remoteId;
	}
	public void setRemoteId(String remoteId)
	{
		this.remoteId = remoteId;
	}

	/**
		The maximum length of the location that can be stored in the database.
		@see #setLocation(String)
	*/
	public static final int MAX_LOCATION_LENGTH = 255;
	private String location;
	/**
		Get the location of the user (if known).
		@since 3.12
		@hibernate.property column="`location`" type="string" length="255" not-null="false" update="false"
	*/
	public String getLocation()
	{
		return location;
	}
	public void setLocation(String location)
	{
		this.location = location;
	}
	
	private Float locationLat;
	/**
		Get the location latitude of the user (if known).
		@since 3.12
		@hibernate.property column="`location_lat`" type="float" not-null="false" update="false"
	*/
	public Float getLocationLatitude()
	{
		return locationLat;
	}
	public void setLocationLatitude(Float locationLat)
	{
		this.locationLat = locationLat;
	}
	
	private Float locationLong;
	/**
		Get the location longitude of the user (if known).
		@since 3.12
		@hibernate.property column="`location_long`" type="float" not-null="false" update="false"
	*/
	public Float getLocationLongitude()
	{
		return locationLong;
	}
	public void setLocationLongitude(Float locationLong)
	{
		this.locationLong = locationLong;
	}

	/**
		The maximum length of the authentication method that can be stored in the database.
		@see #setAuthenticationMethod(String)
	*/
	public static final int MAX_AUTHENTICATION_METHOD = 255;
	private String authenticationMethod;
	/**
		Get the authentication method used for this session.
		@since 3.9
		@hibernate.property column="`authentication_method`" type="string" length="255" not-null="false" update="false"
	*/
	public String getAuthenticationMethod()
	{
		return authenticationMethod;
	}
	public void setAuthenticationMethod(String authenticationMethod)
	{
		this.authenticationMethod = authenticationMethod;
	}

}
