/*
	$Id$

	Copyright (C) 2020 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core.query;

import java.util.Collection;
import java.util.Collections;
import java.util.HashSet;
import java.util.Set;

import net.sf.basedb.core.DbControl;
import net.sf.basedb.util.Values;

/**
	Restriction that filters on a list of ID values:
	
	id IN (a, b, c, ....)
	id NOT IN (a, b, c, ....)
	
	@author nicklas
	@since 3.16
*/
public class IdListRestriction
	implements Restriction
{

	private boolean notIn;
	private Set<Integer> idList;
	private final Expression idExpression;
	private String tmpTable;
	
	public IdListRestriction()
	{
		this(false, new HashSet<>());
	}
	
	/**
		@since 3.18
	*/
	public IdListRestriction(boolean notIn)
	{
		this(notIn, new HashSet<>());
	}
	
	public IdListRestriction(Set<Integer> idList)
	{
		this(false, idList);
	}
	
	/**
		@since 3.18
	*/
	public IdListRestriction(boolean notIn, Set<Integer> idList)
	{
		this.notIn = notIn;
		this.idList = new HashSet<>(idList);
		this.idExpression = Hql.property("id");
	}
	
	@Override
	public String toQl(Query query, DbControl dc) 
	{
		if (tmpTable == null && idList.size() > 1000)
		{
			tmpTable = dc.createTempIdTable(idList);
		}
		String hql = idExpression.toQl(query, dc) + (notIn ? " NOT IN (" : " IN (");
		if (tmpTable != null)
		{
			hql += "native('select [id] from [" + tmpTable + "]')";
		}
		else
		{
			hql += idList.size() == 0 ? "0" : Values.getString(idList, ", ", true);
		}
		return "("+ hql + "))";
	}

	@Override
	public Collection<? extends QueryElement> getChildren()
	{
		return Collections.singleton(idExpression);
	}
	@Override
	public String toString() 
	{
		String idString = idList.size() == 0 ? "0" : Values.getString(idList, ", ", true);
		return idExpression.toString() + (notIn ? " NOT IN " : " IN ") + "(" + idString + ")";
	}

	/**
		Merge this id list with another list. This list will now 
		contain the combined set of ids from both lists.
		NOTE! This method ignore the 'notIn' setting for both lists.
	*/
	public void union(IdListRestriction other)
	{
		this.idList.addAll(other.idList);
	}
	
	/**
		Merge this id list with another list. This list will
		now contain only the ids that are found in both lists.
		NOTE! This method ignore the 'notIn' setting for both lists.
	*/
	public void intersect(IdListRestriction other)
	{
		this.idList.retainAll(other.idList);
	}
	
	/**
		Merge this id list with another list by removing
		ids that are found in the other list.
		NOTE! This method ignore the 'notIn' setting for both lists.
	*/
	public void remove(IdListRestriction other)
	{
		this.idList.removeAll(other.idList);
	}
	
	/**
		Merge this list with another list with AND logic respecting
		the notIn setting. This may cause the list of ids and notIn 
		setting of this list to change.
		@since 3.18
	*/
	public void mergeAnd(IdListRestriction other)
	{
		if (notIn == other.notIn)
		{
			if (notIn)
			{
				// NOT IN(A,B) AND NOT IN(B,C) --> NOT IN(A,B,C)
				// Merge with UNION and keep notIn setting
				union(other);
			}
			else
			{
				// IN(A,B) AND IN(B,C) --> IN(B)
				// Merge with INTERSECT and keep notIn setting
				intersect(other);
			}
		}
		else
		{
			if (notIn)
			{
				// NOT IN(A,B) AND IN(B,C) --> IN(C)
				// Swap, merge with REMOVE and change notIn setting
				Set<Integer> tmp = new HashSet<>(other.idList);
				tmp.removeAll(idList);
				idList = tmp;
				notIn = false;
			}
			else
			{
				// IN(A,B) AND NOT IN(B,C) --> IN(A)
				// Merge with REMOVE and keep notIn setting
				remove(other);
			}
		}
	}
	
}
