/**
	$Id: AnnotationSetSnapshot.java 7610 2019-02-27 13:18:43Z nicklas $

	Copyright (C) 2009 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core.snapshot;

import java.io.IOException;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.Date;
import java.util.List;

import net.sf.basedb.core.Annotatable;
import net.sf.basedb.core.AnnotationSet;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.Item;
import net.sf.basedb.core.data.AnnotationData;
import net.sf.basedb.core.data.AnnotationSetData;

/**
	A snapshot of an annotation set that is used to store 
	information as a file. This will make it quicker to load
	annotation values than if we have to do it from the database
	each time. The snapshot will store information about:
	<ul>
	<li>The item the annotation set is linked to
	<li>The primary annotations (including their values)
	<li>Unit the values should be displayed in (values are 
		always stored in the unit of the annotation type)
	<li>Inherited annotations (not including the values)
	<li>Inherited annotation sets (not including the values)
	</ul>
	
	The reason that the snapshot doesn't include values for inherited
	annotations is that it is better to just store a reference to 
	the annotation set the annotations are inherited from. The values
	can then be loaded from the referenced snapshot. This design makes it
	easier to "invalidate" (eg. delete the serialized file) a cached snapshot 
	when an annotation has been modified. We only have to delete the file
	that is associated with the primary annotation set.
	<p>
	<b>IMPORTANT NOTE TO DEVELOPERS!!</b>
	<p>
	Do not forget to synchronize all serialization code and to increment
	the {@link #FILE_VERSION} in case the number of variables that needs
	serialization in this class or in the {@link AnnotationSnapshot} changes. 

	@author Nicklas
	@version 2.14
	@base.modified $Date: 2019-02-27 14:18:43 +0100 (Wed, 27 Feb 2019) $
*/
public class AnnotationSetSnapshot
	implements Serializable
{

	private static final long serialVersionUID = -7092844726985995667L;

	/**
		The version number for the file format in use when serializing
		this data. If the format changes in an incompatible way, this
		number must be increased. This will make sure that the old files
		are not used and that new files are generated instead.
	*/
	public static final int FILE_VERSION = Integer.valueOf(5).intValue();
	
	private transient long created;
	private transient int annotationSetId;
	private transient int itemId;
	private transient Item itemType;
	private transient List<AnnotationSnapshot> annotations;
	
	/**
		Creates a new, empty snapshot.
	*/
	public AnnotationSetSnapshot()
	{}
	
	/**
		Initialise the snapshot with data from the given annotation 
		set, including values for all primary annotations and information
		about inherited annotations.
	*/
	public void init(AnnotationSetData set)
	{
		this.created = System.currentTimeMillis();
		this.annotationSetId = set.getId();
		this.itemId = set.getItemId();
		this.itemType = Item.fromValue(set.getItemType());
		this.annotations = new ArrayList<AnnotationSnapshot>();

		for (AnnotationData a : set.getAnnotations())
		{
			AnnotationSnapshot as = new AnnotationSnapshot();
			as.init(a);
			annotations.add(as);
		}
		
		// Sort annotations so that project-specific annotation comes before default values
		// This is important so that the SnapshotManager can 
		// decide if a default value should be ignored or not
		// See SnapshotManager.findAnnotations()
		Collections.sort(annotations, new SortByProjectComparator());
	}
	
	
	/**
		Get the date and time this snapshot was created.
	*/
	public Date getSnapshotDate()
	{
		return new Date(created);
	}
	
	
	/**
		Get the ID of the item the annotations in this snapshot 
		belong to.
	*/
	public int getItemId()
	{
		return itemId;
	}
	
	/**
		Get item the annotations belong to.
		@param dc A DbControl to use for database access
	*/
	public Annotatable getItem(DbControl dc)
	{
		return itemType.getById(dc, itemId);
	}
	
	/**
		Get the item type of the item the annotations in this snapshot
		belong to.
	*/
	public Item getItemType()
	{
		return itemType;
	}

	/**
		Get the number of annotations.
	*/
	public int getSize()
	{
		return annotations.size();
	}
	
	/**
		Get a list with the all annotations in this snapshot.
	*/
	List<AnnotationSnapshot> getAnnotations()
	{
		return annotations;
	}
	
	/**
		Get the id of the annotation set this is a snapshot of.
	*/
	public int getAnnotationSetId()
	{
		return annotationSetId;
	}
	
	/**
		Get the annotation set this is a snapshot of.
	*/
	public AnnotationSet getAnnotationSet(DbControl dc)
	{
		return AnnotationSet.getById(dc, annotationSetId);
	}
	
	private void writeObject(java.io.ObjectOutputStream out)
		throws IOException
   {
		out.writeLong(created);
		out.writeInt(annotationSetId);
		out.writeInt(itemId);
		out.writeInt(itemType.getValue());
		out.writeInt(annotations == null ? 0 : annotations.size());
		if (annotations != null)
		{
			for (AnnotationSnapshot tmp : annotations)
			{
				tmp.serializeObject(out);
			}
		}
   }
	private void readObject(java.io.ObjectInputStream in)
		throws IOException, ClassNotFoundException
	{
		created = in.readLong();
		annotationSetId = in.readInt();
		itemId = in.readInt();
		itemType = Item.fromValue(in.readInt());
		int numAnnotations = in.readInt();
		annotations = new ArrayList<AnnotationSnapshot>(numAnnotations);
		for (int i = 0; i < numAnnotations; ++i)
		{
			AnnotationSnapshot tmp = new AnnotationSnapshot();
			tmp.deserializeObject(in);
			annotations.add(tmp);
		}
	}
	
	/**
		Comparator for sorting annotation snapshots so that
		project-specific annotations appear before default
		annotations. This makes it easier to filter out
		default values that has been overridden by projec-specific values.
		@since 3.10
	*/
	static class SortByProjectComparator
		implements Comparator<AnnotationSnapshot>
	{
		SortByProjectComparator() 
		{}

		@Override
		public int compare(AnnotationSnapshot o1, AnnotationSnapshot o2) 
		{
			int pId1 = o1.getProjectId();
			int pId2 = o2.getProjectId();
			// IMPORTANT! Annotations with a project comes before default annotations
			if (pId1 != pId2) return pId2 - pId1; 
			
			return o2.getThisAnnotationId() - o1.getThisAnnotationId();
		}
	}
}
