/*
 $Id: BioAssaySetUtil.java 6924 2015-06-03 06:19:32Z nicklas $

 Copyright (C) 2006 Johan Enell, Nicklas Nordborg, Martin Svensson

 This file is part of BASE - BioArray Software Environment.
 Available at http://base.thep.lu.se/

 BASE is free software; you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 3 of the License, or
 (at your option) any later version.

 BASE is distributed in the hope that it will be useful, but
 WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with BASE. If not, see <http://www.gnu.org/licenses/>.
 */
package net.sf.basedb.util;

import net.sf.basedb.core.AnnotationType;
import net.sf.basedb.core.BaseException;
import net.sf.basedb.core.BioAssay;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.DynamicQuery;
import net.sf.basedb.core.DynamicSpotQuery;
import net.sf.basedb.core.Include;
import net.sf.basedb.core.InvalidDataException;
import net.sf.basedb.core.InvalidUseOfNullException;
import net.sf.basedb.core.ItemQuery;
import net.sf.basedb.core.RawBioAssay;
import net.sf.basedb.core.Metadata.PropertyPath;
import net.sf.basedb.core.RootRawBioAssay;
import net.sf.basedb.core.query.Expression;
import net.sf.basedb.core.query.Restriction;
import net.sf.basedb.core.snapshot.AnnotationSnapshot;
import net.sf.basedb.core.snapshot.AnnotationTypeFilter;
import net.sf.basedb.core.snapshot.SnapshotManager;
import net.sf.basedb.util.filter.Filter;
import net.sf.basedb.util.jep.Jep;

import java.util.Set;
import java.util.TreeSet;

/**
	This is a utility class for {@link net.sf.basedb.core.BioAssaySet}.

	@base.modified $Date: 2015-06-03 08:19:32 +0200 (Wed, 03 Jun 2015) $
	@author Johan
	@version 2.0
*/
public class BioAssaySetUtil
{

	/**
		Find the annotation values for a given annotation type on a bioassay.
		This method will only check annotations, primary and inherited, on the
		root raw bioassays that the bioassay is a child of. 
		<p> 
		Since a bioassay may
		have more than one parent raw bioassay and each annotation may have
		multiple values it is possible that the set contains multiple values.
		Each value is only present once regardless of how many times it appears
		in the annotations.
		
		@param dc The DbControl for database access
		@param snapshotManager A snapshot manager that is used for loading the annotations
			from either the database or the file cache (if null, an internal, non-reusable
			snapshot manager is used)
		@param bioAssay The bioassay to get the annotations for
		@param annotationType The annotation type of the annotations
		@return A set containing all values
		@since 2.14
	*/
	public static Set<?> getAnnotationValues(DbControl dc, SnapshotManager snapshotManager, 
		BioAssay bioAssay, AnnotationType annotationType)
	{
		Set<Object> allValues = new TreeSet<Object>();
		if (snapshotManager == null) snapshotManager = new SnapshotManager();
		Filter<AnnotationSnapshot> filter = new AnnotationTypeFilter(annotationType);
		for (AnnotationSnapshot a : snapshotManager.findAnnotations(dc, bioAssay, filter, true))
		{
			if (a != null) allValues.addAll(a.getActualValues());
		}
		ItemQuery<RootRawBioAssay> rbaQuery = bioAssay.getRootRawBioAssays();
		rbaQuery.include(Include.MINE, Include.SHARED, Include.IN_PROJECT, Include.OTHERS);
		for (RootRawBioAssay rb : rbaQuery.list(dc))
		{
			for (AnnotationSnapshot a : snapshotManager.findAnnotations(dc, rb, filter, true))
			{
				if (a != null) allValues.addAll(a.getActualValues());
			}
		}
		return allValues;
	}

	/**
		Find the values of parent properties for a given bioassay. This method
		starts by loading all raw bioassays that are parents to the bioassay.
		For each raw bioassay it uses the {@link PropertyPath} 
		object to find the value of the given property for that raw bioassay.
		<p> 
		Since a bioassay may have more than one parent raw bioassay and it is possible 
		that the set contains multiple values. Each value is only present once 
		regardless of how many times it appears among the parents.
		<p>
		Examples of property paths:
		<ul>
		<li>spots: to get the number of spots in the raw bioassay
		<li>software.name: to get the name of the software used to analyse the image files
		<li>scan.hybridization.creationEvent.eventDate: to get the date when the hybridization
			was created
		</ul>
		
		@param dc The DbControl for database access
		@param bioAssay The bioassay to get the annotations for
		@param property The property to get the value for
		@return A set containing all values
	*/
	public static Set<Object> getParentProperties(DbControl dc, BioAssay bioAssay, PropertyPath<RawBioAssay, ?> property)
	{
		ItemQuery<RawBioAssay> rbaQuery = bioAssay.getRawBioAssays();
		rbaQuery.include(Include.MINE, Include.SHARED, Include.IN_PROJECT, Include.OTHERS);
		Set<Object> allValues = new TreeSet<Object>();
		for (RawBioAssay rb : rbaQuery.list(dc))
		{
			Object value = property.getValue(dc, rb);
			if (value == null) value = "Unknown";
			allValues.add(value);
		}
		return allValues;
	}
		
	/**
		Parse a JEP expression and create a restrictions that is suitable to
		be used in a dynamic query. In addition to the standard functions defined
		by JEP (see {@link Jep#getFunctions()}) this method can use additional 
		functions. The extra functions are defined by the query. See
		{@link DynamicQuery#getJepFunctions(DbControl, boolean)} and it's subclasses
		for detailed information. Here are some useful functions:

		<ul>
		<li>all functions defined by the {@link #createJepExpression(DbControl, 
			String, DynamicQuery)} method
		<li>inList(list): Checks if the reporter is included in the specified list
		<li>notInList(list): Checks if the reporter isn't included in the specified list
		</ul>
	 	@param dc DbControl to use when accessing the database. 
	 	@param formula The formula to parse. Must not be null.
	 	@param query The query where the restriction will be used, it is not
	 		recommended to use the expression in a different query since it may
	 		generate incorrect SQL
	 	@return A restriction to be used on a query.
	 	@throws InvalidDataException  If any of required parameters are null.
	 	@throws BaseException If creating the restriction fails.
		@see net.sf.basedb.util.jep.Jep
	*/
	public static Restriction createJepRestriction(DbControl dc, String formula, DynamicQuery query)
	{
		if (query == null) throw new InvalidUseOfNullException("query");
		if (formula == null) throw new InvalidUseOfNullException("formula");
		return Jep.formulaToRestriction(formula, query.getJepFunctions(dc, true));		
	}
	
	/**
		Parse a JEP expression and create a restrictions that is suitable to
		be used in a dynamic query. In addition to the standard functions defined
		by JEP (see {@link Jep#getFunctions()}) this method can use additional 
		functions. The extra functions are defined by the query. See
		{@link DynamicQuery#getJepFunctions(DbControl, boolean)} and it's subclasses
		for detailed information. Here are some useful functions:
		
		<ul>
		<li>raw(property): The value of the specified raw data property. The use of this function
			can be denied if the bioassayset contains merged spot data. The 
			{@link DynamicSpotQuery#setDisableRawJoinOnMergedData(boolean)}
			flag controls this.
		<li>rep(property): The value of the specified reporter property
		<li>pos(): The position of a spot. Note that this is not the same as raw(position)
		<li>ch(n): The intensity in channel n (un-transformed)
		<li>rawCh(n): The possibly transformed intensity in channel n
		<li>score(list): The score of the {@link net.sf.basedb.core.data.ReporterData} in the 
			specified {@link net.sf.basedb.core.ReporterList} (numeric or external ID)
		<li>xtra(extraValue): The value of the specified {@link net.sf.basedb.core.ExtraValue} 
			(numeric ID or the external ID of the {@link net.sf.basedb.core.ExtraValueType})
		</ul>
	  	@param dc The DbControl to use when accessing the database. 
	 	@param formula The formula that should be parsed. Must not be null.
	 	@param query The query where the expression will be used, it is not
	 		recommended to use the expression in a different query since it may
	 		generate incorrect SQL
	 	@return An expression to be used with a query.
	 	@throws InvalidDataException If any of required parameters are null.
	 	@throws BaseException If creating the expression fails.
		@see net.sf.basedb.util.jep.Jep
		@since 2.15
	*/
	public static Expression createJepExpression(DbControl dc, String formula, DynamicQuery query)
	{
		if (query == null) throw new InvalidUseOfNullException("query");
		if (formula == null) throw new InvalidUseOfNullException("formula");
		return Jep.formulaToExpression(formula, query.getJepFunctions(dc, false));		
	}
	
}
