package net.sf.basedb.util.charset;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.nio.charset.Charset;
import java.nio.charset.CharsetDecoder;

import net.sf.basedb.util.FileUtil;
import net.sf.basedb.util.InputStreamTracker;

/**
	Utility class for testing if a text stream can be parsed using 
	a given character set. There are two sides of the testing:
	
	* The technical side which checks for invalid byte sequences, etc. This works well for
	  UTF-8 but it is, for example, not able to discriminate betwee different ISO-8859-? or 
	  Windows-? encoding.
	* The content side which can check that the parsed content contains some expected text 
	  strings. This can be used to discriminate between diffent ISO-8859-? or Windows-?
	  encoding by using careful choices of text strings to look for.
	
	
	@author nicklas
	@since 3.15
*/
public class CharsetDetector 
{

	private final Charset charset;
	private final StringDetector lineTester;
	private IOException parsingFailure;
	private long parsedBytes;
	private int parsedLines;
	
	/**
		Create a detector for the given character set that
		only detects technical issues. Useful for UTF-8.
	*/
	public CharsetDetector(Charset charset)
	{
		this(charset, null);
	}

	/**
		Create a detector for the given character set that uses technical
		an content-based detection. If no lineTester is given it will use
		only technical detection.
	*/
	public CharsetDetector(Charset charset, StringDetector lineTester)
	{
		this.charset = charset;
		this.lineTester = lineTester;
	}
	
	/**
		Get the character set this detector is configured to use.
	*/
	public Charset getCharset()
	{
		return charset;
	}
	
	/**
		Test if the given input stream can be parsed with the configured character set.
		The stream is read until the end is reached or until there is a decoding failure.
	*/
	public boolean testIt(InputStream in)
	{
		return testIt(in, -1, -1);
	}
	
	/**
		Test if the given input stream can be parsed with the configured character set.
		The stream is read until maxBytes bytes has been parsed or until there is a decoding failure.
		@param maxBytes Max number of bytes to parse or -1 to not use a limit
		@param maxLines Max number of lines to parse or -1 to not use a limit
	*/
	public boolean testIt(InputStream in, long maxBytes, int maxLines)
	{
		CharsetDecoder dec = charset.newDecoder();
		InputStreamTracker tracker = new InputStreamTracker(in);
		BufferedReader tryit = new BufferedReader(new InputStreamReader(tracker, dec));

		parsingFailure = null;
		parsedBytes = 0;
		parsedLines = 0;
		try
		{
			String line = null;
			boolean lineCheck = false;
			do
			{
				line = tryit.readLine();
				parsedLines++;
				parsedBytes = tracker.getNumRead();
				if (line != null && lineTester != null) 
				{
					lineCheck = lineTester.checkLine(parsedLines, line);
					if (lineCheck) break;
				}
			} while (line != null && (maxBytes == -1 || parsedBytes < maxBytes) && (maxLines == -1 || parsedLines < maxLines));
			
			// We reached the end but the lineTester never returned TRUE so we call lineTester.eof()
			if (lineTester != null && !lineCheck)
			{
				 lineTester.eof(parsedLines);
			}
			
		}
		catch (IOException ex)
		{
			parsingFailure = ex;
		}
		catch (RuntimeException ex)
		{
			parsingFailure = new IOException(ex);
		}
		finally
		{
			FileUtil.close(tryit);
		}
		return parsingFailure == null;
	}
	
	/**
		Get the number of bytes that the last test operation parsed.
	*/
	public long getParsedBytes()
	{
		return parsedBytes;
	}
	
	public int getParsedLines()
	{
		return parsedLines;
	}
	
	/**
		If the last test failed, get the exception that was thrown by the parser.
	*/
	public IOException getParsingFailure()
	{
		return parsingFailure;
	}
	
}
