package net.sf.basedb.util.charset;

import java.nio.charset.Charset;
import java.nio.charset.spi.CharsetProvider;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;

import net.sf.basedb.core.Config;
import net.sf.basedb.util.Enumeration;
import se.raek.charset.Utf8WithFallbackCharsetProvider;

/**
	Helper class for working with charsets and enabling the UTF-8 with fallback charsets:
	
	* X-UTF-8_with_ISO-8859-1_fallback 
	* X-UTF-8_with_windows-1252_fallback 
	
	@author nicklas
	@since 3.15
*/
public class CharsetUtil 
{

	/**
		The provider that is providing the UTF-8 with fallback characeter sets.
	*/
	public static final CharsetProvider UTF8_WITH_FALLBACK = new Utf8WithFallbackCharsetProvider();
	
	/**
		Get a character set by name. This will first look if it is a UTF-8 with fallback charset
		and if not check the system {@link Charset#forName(String)}.
	*/
	public static Charset getCharset(String name)
	{
		if (name == null) name = Config.getCharset();
		Charset cs = UTF8_WITH_FALLBACK.charsetForName(name);
		return cs == null ? Charset.forName(name) : cs;
	}
	
	/**
		Get all charsets that {@link Config#getAllCharsets()} returns and optionally include
		the fallback charsets. Note that the fallback charsets are automatically sorted just
		below the UTF-8 character set.
	*/
	public static List<String> getAllCharsets(boolean includeFallbacks)
	{
		List<String> charsets = new ArrayList<>(Config.getAllCharsets());
		
		if (includeFallbacks)
		{
			int utf8 = charsets.indexOf("UTF-8")+1;
			if (utf8 > 0)
			{
				Iterator<Charset> it = UTF8_WITH_FALLBACK.charsets();
				while (it.hasNext())
				{
					charsets.add(utf8++, it.next().name());
				}
			}
		}
		return charsets;
	}
	
	/**
		Convert charset names to an enumeration with display-friendly key/title
		pairs.
	*/
	public static Enumeration<String, String> asEnumeration(Collection<String> charsets)
	{
		Enumeration<String, String> e = new Enumeration<String, String>();
		for (String name : charsets)
		{
			Charset cs = getCharset(name);
			e.add(cs.name(), cs.displayName());
		}
		e.lock();
		return e;
	}
	
}
