/**
	$Id:ExtensionPoint.java 4187 2008-03-20 11:15:25Z nicklas $

	Copyright (C) Authors contributing to this file.

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.util.extensions;

import net.sf.basedb.util.extensions.xml.XmlLoader;


/**
	An extension point is a place, defined by a client application, that
	can be extended. Each extension point is identified by an ID which
	must be unique. We recommend that ID:s are generated in the same
	way as Java package names. An extension point must also define
	what kind of {@link Action} it is possible to add to it. This
	is done with the {@link #getActionClass()}. It is also possible
	to define a default render class and if an extension is allowed
	to override it or not.
	
	<p>
	Extension points can, for example, be created:
	<ul>
	<li>Programmatically, using {@link ExtensionPointBean} object
	<li>Loaded from extension definition XML files using a {@link XmlLoader}.
	<li>Or, in any other way by providing a custom implementation of this interface
	</ul>

	<p>
	Before extension points can be used the must be registered with
	{@link Registry#registerExtensionPoint(ExtensionPoint, ClassLoader)}. Extensions to
	an extension point are registered in the same registry with 
	{@link Registry#registerExtension(Extension, ClassLoader)}. To load and use all
	registered extensions use 
	{@link Registry#useExtensions(ClientContext, ExtensionsFilter, String...)}.

	@author nicklas
	@version 2.7
	@base.modified $Date:2008-03-20 12:15:25 +0100 (Thu, 20 Mar 2008) $
	@see ExtensionPointBean
	@see XmlLoader
*/
public interface ExtensionPoint<A extends Action>
{
	
	/**
		Get the ID of the extension point.
		@return A non-null unique identifier value
	*/
	public String getId();
	
	/**
		Get the name of the extension point. This is just for display
		purposes and the value is optional. If no name is given the
		ID can be used instead.
	*/
	public String getName();
	
	/**
		Get a description of the extension point. This value is optional
		but we recommend that the description contains a text documenting
		the extension point and has information that is useful for everyone
		who wants to implement extensions for the extension point. 
		This should include context specific information.
		@return An optional description
	*/
	public String getDescription();
	
	/**
		Get the class object that represents the type of action that
		can be added to this extension point. The class must implement
		the {@link Action} interface. Extensions that are extending this
		extension point must provide an {@link ActionFactory} that is capable
		of creating instances of the specified class.
		
		@return The class of the actions that can be used on this extension point
			(required)
	*/
	public Class<A> getActionClass();
	
	/**
		This is an optional factory for creating {@link Renderer} 
		instances. Renderers are not a required part of the extension 
		system, but will make it easier to write the code for extension
		points.
		
		@return A renderer factory or null
	*/
	public RendererFactory<? super A> getRendererFactory();
	
	/**
		If an extension is allowed to override the renderer factory with an
		implementation of its own. This can be seen as a possibility where
		extensions can create their own look and feel. If strict compliance is
		needed overriding should of course not be allowed.
		@return TRUE to allow extensions to override, FALSE otherwise
	*/
	public boolean allowRendererOverride();
	
	/**
		Get the error handler factory that should be used with this
		extension point. 
		@return An error handler factory, or null to use the system default
		@since 2.17
	*/
	public ErrorHandlerFactory<? super A> getErrorHandlerFactory();
}
