/**
	$Id: ExtensionPointFilter.java 7224 2016-11-14 12:43:01Z nicklas $

	Copyright (C) 2011 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.util.extensions.xml;

import java.util.regex.Pattern;

import org.jdom2.Element;

import net.sf.basedb.util.Values;
import net.sf.basedb.util.filter.Filter;

/**
	Filter implementation that can be used to match extension
	points and extensions based on the ID of the extension point.
	<p>
	
	Extensions points will pass the filter if their ID matches the
	given pattern.
	<p>
	
	Extensions will pass the filter if the ID of the extension point
	they are extending matches the given pattern.
	
	@author Nicklas
	@since 3.0
	@base.modified $Date: 2016-11-14 13:43:01 +0100 (Mon, 14 Nov 2016) $
*/
public class ExtensionPointFilter
	implements Filter<Element>
{

	private static final org.slf4j.Logger log = 
		org.slf4j.LoggerFactory.getLogger(ExtensionPointFilter.class);
	
	private final Pattern pattern;
	
	/**
		Create a new filter based on the given pattern.
		@param pattern A pattern, null is not allowed
	*/
	public ExtensionPointFilter(Pattern pattern)
	{
		if (pattern == null) throw new NullPointerException("pattern");
		this.pattern = pattern;
	}
	
	/**
		Create a new filter based on the given pattern.
		
		@param pattern A pattern that will be used in {@link Pattern#compile(String)}, null is not allowed
	*/
	public ExtensionPointFilter(String pattern)
	{
		this(Pattern.compile(pattern));
	}
	
	/*
		From the Filter interface
		--------------------------
	*/
	@Override
	public boolean evaluate(Element tag)
	{
		String tagName = tag.getName();
		String idToMatch = null;
		if ("extension-point".equals(tagName))
		{
			// The ID for extension point is in "id" attribute, but...
			idToMatch = tag.getAttributeValue("id");
			
			// ...it may not be the full id, check top-level "id-base" attribute and prepend it if needed
			String idBase = Values.getStringOrNull(tag.getDocument().getRootElement().getAttributeValue("id-base"));
			if (idBase != null) idToMatch = idBase + idToMatch;
		}
		else if ("extension".equals(tagName))
		{
			// The ID for extension point is in "extends" attribute -- no "id-base" needed here
			idToMatch = Values.getStringOrNull(tag.getAttributeValue("extends"));
		}
		else if ("ref".equals(tagName))
		{
			// The ID for extension point is in the text node -- no "id-base" needed here
			idToMatch = Values.getStringOrNull(tag.getText());
		}
		boolean matches = idToMatch != null && pattern.matcher(idToMatch).matches();
		
		if (log.isDebugEnabled()) 
		{
			if (matches)
			{
				log.debug(tagName + "[" + idToMatch + "] matches filter: " + pattern);
			}
			else
			{
				log.debug(tagName + "[" + idToMatch + "] doesn't match filter: " + pattern);
			}
		}

		return matches;
	}
	// --------------------------

	@Override
	public String toString() 
	{
		return "ExtensionPointFilter["+pattern+"]";
	}
	
	
	
}
