/**
	$Id$

	Copyright (C) 2015 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.util.formatter;

import java.util.Date;
import java.util.List;

import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.Item;
import net.sf.basedb.core.ItemNotFoundException;
import net.sf.basedb.core.Nameable;
import net.sf.basedb.core.Operator;
import net.sf.basedb.core.PropertyFilter;
import net.sf.basedb.core.RawDataType;
import net.sf.basedb.core.RawDataTypes;
import net.sf.basedb.core.Type;
import net.sf.basedb.util.Values;

/**
 	Formatter for PropertyFilter items. This will create a more readable representation
 	of the property filter than the {@link PropertyFilter#toString()} method. It knows
 	about some special properties and load the names of annotation types, item subtypes, etc
 	instead of just displaying their ID values.
 	
  	@author Nicklas
  	@since 3.5
 */
public class PropertyFilterFormatter 
	implements Formatter<PropertyFilter>
{

	private final DbControl dc;
	private final Formatter<Date> dateFormatter;
	private final boolean tagElements;
	
	/**
		Create a new formatter.
	*/
	public PropertyFilterFormatter(DbControl dc, Formatter<Date> dateFormatter)
	{
		this(dc, dateFormatter, false);
	}
	
	/**
		Create a new formatter with option to create HTML-like tags around elements. The
		following tags are used: &lt;value&gt;, &lt;name&gt;, &lt;operator&gt;, &lt;function&gt;
		@since 3.13.1
	*/
	public PropertyFilterFormatter(DbControl dc, Formatter<Date> dateFormatter, boolean tagElements)
	{
		this.dc = dc;
		this.dateFormatter = dateFormatter;
		this.tagElements = tagElements;
	}
	
	/*
		From the Formatter interface
		-------------------------------------------
	*/
	@Override
	public String format(PropertyFilter filter)
	{		
		String property = filter.getProperty();
		Operator operator = filter.getOperator();
		String value = filter.getValue();
		String quote = filter.getValueType().isNumerical() || value == null ? "" : "'";
		
		StringBuilder sb = new StringBuilder();
		
		int numHops = 0;
		while (property.startsWith("/") && numHops < 2)
		{
			// Parent or child item filter
			numHops++;
			String[] tmp = property.split("/", 5);
			int baseIndex = 0;
			String parentOrChild = "parent";
			if (tmp.length == 5)
			{
				baseIndex = 1;
				parentOrChild = tmp[1].toLowerCase();
			}
			
			Item relatedItemType = Item.valueOf(tmp[baseIndex+1]);
			property = tmp[baseIndex+3];
			sb.append(function(parentOrChild)).append("(");
			if (relatedItemType == Item.RAWBIOASSAY)
			{
				RawDataType rdt = RawDataTypes.getRawDataType(tmp[baseIndex+2]);
				if (rdt == null)
				{
					sb.append(value(missing(tmp[baseIndex+2], "This raw data type doesn't exist!")));
				}
				else
				{
					sb.append(value(rdt.getName()));
				}
			}
			else
			{
				int subtypeId = Values.getInt(tmp[baseIndex+2]);
				sb.append(value(nameOfItem(dc, Item.ITEMSUBTYPE, subtypeId, false)));
			}
			sb.append(")");
			if (!property.startsWith("#")) sb.append(".");
		}
		if (property.startsWith("|"))
		{
			// Linked item filter
			String[] tmp = property.split("\\|", 4);
			sb.append(function("link")).append("(");
			sb.append(value(tmp[1]));
			if (tmp.length > 2)
			{
				sb.append(", ").append(value(tmp[2]));
			}
			sb.append(")");
			if (tmp.length > 3)
			{
				sb.append(".");
				property = tmp[3];
			}
			else
			{
				property = "";
			}
		}
		
		if (property.startsWith("#"))
		{
			// Annotation type filter
			if (property.contains("("))
			{
				// The annotation is on a joined property: #(path.to.join)id
				int joinStart = property.indexOf('(');
				int joinEnd = property.indexOf(')');
				sb.append(name(property.substring(joinStart+1, joinEnd)));
				property = property.substring(0, joinStart) + property.substring(joinEnd+1); // Should give us: #id
			}
			
			int startIndex = property.startsWith("##") ? 2 : 1;
			int annotationTypeId = Values.getInt(property.substring(startIndex));
			sb.append("{").append(name(nameOfItem(dc, Item.ANNOTATIONTYPE, annotationTypeId, false))).append("}");
			sb.append(" ").append(operator(operator.toString())).append(" ");
			if (operator.isListOperator() && value != null)
			{
				appendListOfValues(sb, value, quote);
			}
			else
			{
				sb.append(value(quote+value+quote));
			}
		}
		else if (property.startsWith("¤"))
		{
			// Data file type filter
			int i = property.indexOf('(');
			int fileTypeId = Values.getInt(property.substring(1, i));
			String fileFilter = property.substring(i+1, property.length()-1);
			sb.append("{").append(name(nameOfItem(dc, Item.DATAFILETYPE, fileTypeId, false)+"#"+fileFilter)).append("}");
			sb.append(" ").append(operator(operator.toString())).append(" ");
			if (operator.isListOperator() && value != null)
			{
				appendListOfValues(sb, value, quote);
			}
			else
			{
				sb.append(value(quote+value+quote));
			}
		}
		else if (property.startsWith("§"))
		{
			// Item list filter
			if (operator.isNegationOperator()) sb.append(operator("NOT "));
			sb.append(function("memberOfList")).append("(");			
			List<Object> listIds = filter.getValuesAsObjects();
			String sep = "";
			for (Object listId : listIds)
			{
				sb.append(sep);
				sep = ", ";
				sb.append(value(nameOfItem(dc, Item.ITEMLIST, (Integer)listId, true)));
			}
			sb.append(")");
		}
		else if (property.equals("bioWell.row") || property.equals("bioWell.column"))
		{
			WellCoordinateFormatter formatter = new WellCoordinateFormatter(property.equals("bioWell.row"));
			sb.append(name(property)).append(" ").append(operator(operator.toString())).append(" ");
			List<Object> selected = filter.getValuesAsObjects();
			if (selected.size() > 1) sb.append("(");
			if (selected.size() == 0) sb.append("null");
			String sep = "";
			for (Object index : selected)
			{
				sb.append(sep);
				sep = ", ";
				sb.append(value(formatter.format((Integer)index)));
			}
			if (selected.size() > 1) sb.append(")");
		}
		else if (property.equals("parentType"))
		{
			sb.append(name(property)).append(" ").append(operator(operator.toString())).append(" ");
			List<Object> selected = filter.getValuesAsObjects();
			if (selected.size() > 1) sb.append("(");
			if (selected.size() == 0) sb.append("null");
			String sep = "";
			for (Object index : selected)
			{
				sb.append(sep);
				sep = ", ";
				sb.append(value(Item.fromValue((Integer)index).toString()));
			}
			if (selected.size() > 1) sb.append(")");
		}
		else
		{
			Item nameOfItemType = null;
			if (property.equals("itemSubtype") || property.endsWith(".itemSubtype"))
			{
				// Special handling to display name of subtype
				nameOfItemType = Item.ITEMSUBTYPE;
			}
			else if (property.equals("platform"))
			{
				// Special handling to display name of platform
				nameOfItemType = Item.PLATFORM;
			}
			if (nameOfItemType != null)
			{
				sb.append(name(property)).append(" ").append(operator(operator.toString())).append(" ");
				List<Object> itemIds = filter.getValuesAsObjects();
				if (itemIds.size() > 1) sb.append("(");
				if (itemIds.size() == 0) sb.append("null");
				String sep = "";
				for (Object id : itemIds)
				{
					sb.append(sep);
					sep = ", ";
					sb.append(value(nameOfItem(dc, nameOfItemType, (Integer)id, true)));
				}
				if (itemIds.size() > 1) sb.append(")");
			}
			else
			{
				if (property.startsWith("&") || property.startsWith("@")) 
				{
					property = property.substring(1);
				}
				else if (property.startsWith("!x."))
				{
					property = property.substring(3);
				}
				sb.append(name(property)).append(" ").append(operator(operator.toString())).append(" ");
				if (operator.isListOperator() && value != null)
				{
					appendListOfValues(sb, value, quote);
				}
				else
				{
					if (filter.getValueType().isTemporal())
					{
						value = dateFormatter.format((Date)Type.DATE.parseString(value));
					}
					sb.append(value(quote+value+quote));
				}
			}
		}
		return sb.toString();
	}

	/**
		@throws UnsupportedOperationException Always
	*/
	@Override
	public PropertyFilter parseString(String value)
	{
		throw new UnsupportedOperationException("parseString: " + value);
	}
	// -------------------------------------------

	
	private String nameOfItem(DbControl dc, Item itemType, int id, boolean quote)
	{
		String q = quote ? "'" : "";
		try
		{
			return q+((Nameable)itemType.getById(dc, id)).getName()+q;
		}
		catch (ItemNotFoundException ex)
		{
			return missing(Integer.toString(id), "This " + itemType.toString().toLowerCase() + " doesn't exist!");
		}
		catch (RuntimeException ex)
		{
			return missing(Integer.toString(id), ex.getMessage());
		}
	}
	
	private void appendListOfValues(StringBuilder sb, String value, String quote)
	{
		String[] values = value.split("\\|");
		if (values.length > 1) sb.append("(");
		String sep = "";
		for (Object v : values)
		{
			sb.append(sep);
			sep = ", ";
			sb.append(value(quote+v+quote));
		}
		if (values.length > 1) sb.append(")");
		
	}
	
	private String value(String value)
	{
		return tagElements ? "<value>"+value+"</value>" : value;
	}
	
	private String name(String name)
	{
		return tagElements ? "<name>"+name+"</name>" : name;
	}
	private String operator(String op)
	{
		return tagElements ? "<operator>"+op+"</operator>" : op;
	}
	private String function(String function)
	{
		return tagElements ? "<function>"+function+"</function>" : function;
	}
	private String missing(String missing, String title)
	{
		return tagElements ? "<missing title=\""+(title!=null ? title :"Missing")+"\">"+missing+"</missing>" : missing;
	}

	
}
