package net.sf.basedb.util.listable;

import java.util.Set;

import net.sf.basedb.core.ItemQuery;
import net.sf.basedb.core.query.Expressions;
import net.sf.basedb.core.query.Hql;
import net.sf.basedb.core.query.Restriction;
import net.sf.basedb.core.query.Restrictions;

/**
	Transformer implementation that is intended to provide a final
	filtering of target items. The transformer is used by wrapping 
	an existing source item transformer implementation. The {@link #transform(TransformContext, Set)}
	method will first call the wrapped transformer to get a list of target
	items and then apply the given filter before returning the result.
	
	@author nicklas
	@since 3.18
*/
public class RestrictionTransformer 
	extends AbstractSourceItemTransformer
{
	private final SourceItemTransformer transformer;
	private final ItemQuery<?> query;
	
	/**
		Create a new transformer and apply the given restriction to items before they are
		returned. An {@link ItemQuery} for the correct target item type is automatically
		created from the given transformer. The restriction is applied to the query
		before it is executed.
	*/
	public RestrictionTransformer(SourceItemTransformer transformer, Restriction restriction)
	{
		super(transformer.getSourceItemType(), transformer.getTargetItemType());
		this.transformer = transformer;
		this.query = transformer.getTargetItemType().getQuery();
		query.restrictPermanent(restriction);
		query.restrictPermanent(Restrictions.in(Hql.property("id"), Expressions.parameter("ids")));
	}
	
	/**
		Create a new transformer and use the given query to find the target items.
		The query is expected to return the same type of items as the transformer
		and must have defined a parameter with the name 'ids' that take a list
		of id values. The list will be populated with the id values returned
		by the transformer.
	*/
	public RestrictionTransformer(SourceItemTransformer transformer, ItemQuery<?> query)
	{
		super(transformer.getSourceItemType(), transformer.getTargetItemType());
		this.transformer = transformer;
		this.query = query;
	}

	
	@Override
	public Set<Integer> transform(TransformContext context, Set<Integer> source) 
	{
		Set<Integer> target = transformer.transform(context, source);
		query.setIncludes(context.getInclude());
		return safeIdList(context.getDbControl(), query, "ids", target);
	}

}
