/*
	$Id: ConstantMapper.java 7668 2019-03-21 11:50:52Z nicklas $
	
	Copyright (C) 2006 Nicklas Nordborg
	Copyright (C) 2007 Johan Enell, Nicklas Nordborg
	
	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/
	
	BASE is free software; you can redistribute it and/or modify it
	under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.
	
	BASE is distributed in the hope that it will be useful, but
	WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.util.parser;

import java.text.NumberFormat;
import java.util.Date;

import net.sf.basedb.core.DateUtil;
import net.sf.basedb.core.Type;
import net.sf.basedb.util.formatter.Formatter;
import net.sf.basedb.util.parser.FlatFileParser.Data;

/**
	A constant string mapper. The <code>getValue()</code> always
	return the same string which is set in the contructor. 

	@author nicklas
	@version 2.0
	@base.modified $Date: 2019-03-21 12:50:52 +0100 (Thu, 21 Mar 2019) $
*/
public class ConstantMapper
	implements Mapper
{
	private final String constant;
	private final NumberFormat parser;
	private final Formatter<Date> dateParser;
	private final boolean nullIfException;
	private boolean needToParseNumeric;
	private boolean needToParseDate;
	private Double asDouble;
	private Long asLong;
	private Date asDate;
	private RuntimeException parseException;
	
	/**
		Create a constant mapper.
	 	@param constant The constant expression. 
	*/
	public ConstantMapper(String constant)
	{
		this(constant, null, false);
	}
	
	/**
		Create a constant mapper using a specific number formatter as it's parser.
		@param constant The constant expression
		@param parser The number format to use or null to use Double.valueOf()
		@param nullIfException If TRUE, the mapper returns null for unparsable numeric
			values, otherwise an excption is thrown
		@since 2.4
	*/
	public ConstantMapper(String constant, NumberFormat parser, boolean nullIfException)
	{
		this(constant, parser, null, nullIfException);
	}
	
	/**
		Create a constant mapper using a specific number formatter as it's parser.
		@param constant The constant expression
		@param parser The number format to use or null to use Double.valueOf()
		@param nullIfException If TRUE, the mapper returns null for unparsable numeric
			values, otherwise an excption is thrown
		@since 3.15
	*/
	public ConstantMapper(String constant, NumberFormat parser, Formatter<Date> dateParser, boolean nullIfException)
	{
		this.constant = constant;
		this.parser = parser;
		this.dateParser = dateParser;
		this.nullIfException = nullIfException;
		this.needToParseNumeric = constant != null;
		this.needToParseDate = constant != null;
	}
	
	public ConstantMapper(Integer constant)
	{
		this.constant = constant.toString();
		this.asLong = constant.longValue();
		this.asDouble = constant.doubleValue();
		this.parser = null;
		this.needToParseNumeric = false;
		this.nullIfException = false;
		this.dateParser = null;
		this.needToParseDate = false;
	}
	
	public ConstantMapper(Float constant)
	{
		this.constant = constant.toString();
		this.asDouble = constant.doubleValue();
		this.asLong = constant.longValue();
		this.parser = null;
		this.needToParseNumeric = false;
		this.nullIfException = false;
		this.dateParser = null;
		this.needToParseDate = false;
	}
	
	public ConstantMapper(Date constant, Formatter<Date> dateParser)
	{
		this.constant = dateParser != null ? dateParser.format(constant) : DateUtil.formatDate(constant);
		this.asDate = constant;
		this.asDouble = null;
		this.asLong = null;
		this.parser = null;
		this.needToParseNumeric = false;
		this.nullIfException = false;
		this.dateParser = dateParser;
		this.needToParseDate = false;
	}
	
	/*
		From the Mapper interface
		-------------------------------------------
	*/
	/**
		@return The string set in the constructor
	*/
	@Override
	@Deprecated
	public String getValue(Data data)
	{
		return constant;
	}
	@Override
	public String getString(Data data)
	{
		return constant;
	}
	@Override
	public Integer getInt(Data data)
	{
		if (needToParseNumeric) parseNumeric();
		if (parseException != null) throw parseException;
		return asLong == null ? null : asLong.intValue();
	}
	@Override
	public Long getLong(Data data)
	{
		if (needToParseNumeric) parseNumeric();
		if (parseException != null) throw parseException;
		return asLong;
	}
	@Override
	public Float getFloat(Data data)
	{
		if (needToParseNumeric) parseNumeric();
		if (parseException != null) throw parseException;
		return asDouble == null ? null : asDouble.floatValue();
	}
	@Override
	public Double getDouble(Data data)
	{
		if (needToParseNumeric) parseNumeric();
		if (parseException != null) throw parseException;
		return asDouble;
	}
	@Override
	public Date getDate(Data data)
	{
		if (needToParseDate) parseDate();
		if (parseException != null) throw parseException;
		return asDate;
	}
	// -------------------------------------------
	
	/*
		From the Object class
		-------------------------------------------
	*/
	@Override
	public String toString()
	{
		return constant;
	}
	// -------------------------------------------

	private void parseNumeric()
	{
		assert constant != null : "constant == null";
		needToParseNumeric = false;
		try
		{
			asDouble = (Double)Type.DOUBLE.parseString(constant, parser, false);
		}
		catch (RuntimeException ex)
		{
			if (!nullIfException) parseException = ex;
		}
		if (asDouble != null)
		{
			asLong = asDouble.longValue();
		}
	}
	
	private void parseDate()
	{
		assert constant != null : "constant == null";
		needToParseDate = false;
		try
		{
			asDate = (Date)Type.DATE.parseString(constant, dateParser, false);
		}
		catch (RuntimeException ex)
		{
			if (!nullIfException) parseException = ex;
		}
	}
}
